from pandas.core.base import ShallowMixin as ShallowMixin
from pandas.core.groupby.base import GroupByMixin as GroupByMixin
from pandas.core.groupby.groupby import GroupBy as GroupBy, _GroupBy
from pandas.core.groupby.grouper import Grouper as Grouper
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex
from pandas.core.indexes.period import PeriodIndex as PeriodIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex as TimedeltaIndex
from typing import Any, Optional, TypeVar, Union, Callable, Tuple, overload

PipeReturn = TypeVar('PipeReturn')

class Resampler(_GroupBy, ShallowMixin):
    groupby: Any = ...
    keys: Any = ...
    sort: bool = ...
    axis: Any = ...
    kind: Any = ...
    squeeze: bool = ...
    group_keys: bool = ...
    as_index: bool = ...
    exclusions: Any = ...
    binner: Any = ...
    grouper: Any = ...
    def __init__(self, obj: Any, groupby: Optional[Any] = ..., axis: int = ..., kind: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def __getattr__(self, attr: str) -> Any: ...
    def __iter__(self) -> Any: ...
    @property
    def obj(self) -> Any: ...
    @property
    def ax(self) -> Any: ...
    @overload
    def pipe(self: Any, func: Union[Callable[..., PipeReturn], Tuple[Callable[..., PipeReturn], str]], *args: Any, **kwargs: Any) -> PipeReturn: ...
    @overload
    def pipe(self: Any, func: PipeReturn, *args: Any, **kwargs: Any) -> PipeReturn: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    apply: Any = ...
    def transform(self, arg: Any, *args: Any, **kwargs: Any) -> Any: ...    # type: ignore
    def pad(self, limit: Optional[Any] = ...) -> Any: ...
    ffill: Any = ...
    def nearest(self, limit: Optional[Any] = ...) -> Any: ...
    def backfill(self, limit: Optional[Any] = ...) -> Any: ...
    bfill: Any = ...
    def fillna(self, method: Any, limit: Optional[Any] = ...) -> Any: ...
    def interpolate(self, method: str = ..., axis: int = ..., limit: Optional[Any] = ..., inplace: bool = ..., limit_direction: str = ..., limit_area: Optional[Any] = ..., downcast: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def asfreq(self, fill_value: Optional[Any] = ...) -> Any: ...
    def std(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def size(self) -> Any: ...
    def count(self) -> Any: ...
    def quantile(self, q: float = ..., **kwargs: Any) -> Any: ...

def f(self: Any, _method: Any = ..., min_count: int = ..., *args: Any, **kwargs: Any) -> Any: ...
def g(self: Any, _method: Any = ..., *args: Any, **kwargs: Any) -> Any: ...
def h(self: Any, _method: Any = ...) -> Any: ...

class _GroupByMixin(GroupByMixin):
    groupby: Any = ...
    def __init__(self, obj: Any, *args: Any, **kwargs: Any) -> None: ...

class DatetimeIndexResampler(Resampler): ...
class DatetimeIndexResamplerGroupby(_GroupByMixin, DatetimeIndexResampler): ...
class PeriodIndexResampler(DatetimeIndexResampler): ...
class PeriodIndexResamplerGroupby(_GroupByMixin, PeriodIndexResampler): ...
class TimedeltaIndexResampler(DatetimeIndexResampler): ...
class TimedeltaIndexResamplerGroupby(_GroupByMixin, TimedeltaIndexResampler): ...

def resample(obj: Any, kind: Optional[Any] = ..., **kwds: Any) -> Any: ...
def get_resampler_for_grouping(groupby: Any, rule: Any, how: Optional[Any] = ..., fill_method: Optional[Any] = ..., limit: Optional[Any] = ..., kind: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class TimeGrouper(Grouper):
    closed: Any = ...
    label: Any = ...
    kind: Any = ...
    convention: Any = ...
    loffset: Any = ...
    how: Any = ...
    fill_method: Any = ...
    limit: Any = ...
    base: Any = ...
    def __init__(self, freq: str = ..., closed: Optional[Any] = ..., label: Optional[Any] = ..., how: str = ..., axis: int = ..., fill_method: Optional[Any] = ..., limit: Optional[Any] = ..., loffset: Optional[Any] = ..., kind: Optional[Any] = ..., convention: Optional[Any] = ..., base: int = ..., **kwargs: Any) -> None: ...

def asfreq(obj: Any, freq: Any, method: Optional[Any] = ..., how: Optional[Any] = ..., normalize: bool = ..., fill_value: Optional[Any] = ...) -> Any: ...



from collections import namedtuple
from pandas._typing import FrameOrSeries as FrameOrSeries
from pandas.core.base import DataError as DataError, SpecificationError as SpecificationError
from pandas.core.construction import create_series_with_explicit_dtype as create_series_with_explicit_dtype
from pandas.core.dtypes.cast import maybe_convert_objects as maybe_convert_objects, maybe_downcast_numeric as maybe_downcast_numeric, maybe_downcast_to_dtype as maybe_downcast_to_dtype
from pandas.core.dtypes.common import ensure_int64 as ensure_int64, ensure_platform_int as ensure_platform_int, is_bool as is_bool, is_dict_like as is_dict_like, is_integer_dtype as is_integer_dtype, is_interval_dtype as is_interval_dtype, is_list_like as is_list_like, is_numeric_dtype as is_numeric_dtype, is_object_dtype as is_object_dtype, is_scalar as is_scalar, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries, NDFrame as NDFrame
from pandas.core.groupby import base as base
from pandas.core.groupby.groupby import GroupBy as GroupBy, get_groupby as get_groupby
from pandas.core.indexes.api import Index as Index, MultiIndex as MultiIndex, all_indexes_same as all_indexes_same
from pandas.core.internals import Block as Block, BlockManager as BlockManager, make_block as make_block
from pandas.core.series import Series as Series
from pandas.plotting import boxplot_frame_groupby as boxplot_frame_groupby
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Callable, FrozenSet, Optional, Type, Union

NamedAgg = namedtuple('NamedAgg', ['column', 'aggfunc'])
AggScalar = Union[str, Callable[..., Any]]
ScalarResult: Any

def generate_property(name: str, klass: Type[FrameOrSeries]) -> Any: ...
def pin_whitelisted_properties(klass: Type[FrameOrSeries], whitelist: FrozenSet[str]) -> Any: ...

class SeriesGroupBy(GroupBy):
    def apply(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def aggregate(self, func: Optional[Any] = ..., *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def transform(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def filter(self, func: Any, dropna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def nunique(self, dropna: bool=...) -> Series: ...
    def describe(self, **kwargs: Any) -> Any: ...
    def value_counts(self, normalize: bool = ..., sort: bool = ..., ascending: bool = ..., bins: Optional[Any] = ..., dropna: bool = ...) -> Any: ...
    def count(self) -> Series: ...  # type: ignore
    def pct_change(self, periods: int = ..., fill_method: str = ..., limit: Optional[Any] = ..., freq: Optional[Any] = ...) -> Any: ...  # type: ignore

class DataFrameGroupBy(GroupBy):
    def aggregate(self, func: Optional[Any] = ..., *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def transform(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def filter(self, func: Any, dropna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def __getitem__(self, key: Any) -> Any: ...
    def count(self) -> Any: ...
    def nunique(self, dropna: bool=...) -> Any: ...
    boxplot: Any = ...



from pandas.core.dtypes.api import *
from pandas.core.dtypes.concat import union_categoricals as union_categoricals
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype, DatetimeTZDtype as DatetimeTZDtype, IntervalDtype as IntervalDtype, PeriodDtype as PeriodDtype



from typing import Any

def radd(left: Any, right: Any) -> Any: ...
def rsub(left: Any, right: Any) -> Any: ...
def rmul(left: Any, right: Any) -> Any: ...
def rdiv(left: Any, right: Any) -> Any: ...
def rtruediv(left: Any, right: Any) -> Any: ...
def rfloordiv(left: Any, right: Any) -> Any: ...
def rmod(left: Any, right: Any) -> Any: ...
def rdivmod(left: Any, right: Any) -> Any: ...
def rpow(left: Any, right: Any) -> Any: ...
def rand_(left: Any, right: Any) -> Any: ...
def ror_(left: Any, right: Any) -> Any: ...
def rxor(left: Any, right: Any) -> Any: ...



from typing import Any, Optional

gcsfs: Any

def get_filepath_or_buffer(filepath_or_buffer: Any, encoding: Optional[Any] = ..., compression: Optional[Any] = ..., mode: Optional[Any] = ...) -> Any: ...



from pandas._libs import  Timedelta as Timedelta
from pandas.core.accessor import delegate_names as delegate_names
from pandas.core.arrays import datetimelike as dtl
from pandas.core.arrays.timedeltas import TimedeltaArray as TimedeltaArray

from pandas.core.dtypes.missing import is_valid_nat_for_dtype as is_valid_nat_for_dtype, isna as isna
from pandas.core.indexes.base import Index as Index, maybe_extract_name as maybe_extract_name
from pandas.core.indexes.datetimelike import DatetimeIndexOpsMixin as DatetimeIndexOpsMixin, DatetimeTimedeltaMixin as DatetimeTimedeltaMixin, DatetimelikeDelegateMixin as DatetimelikeDelegateMixin
from pandas.core.indexes.extension import inherit_names as inherit_names
from pandas.tseries.frequencies import to_offset as to_offset
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Optional

class TimedeltaDelegateMixin(DatetimelikeDelegateMixin): ...

class TimedeltaIndex(DatetimeTimedeltaMixin, dtl.TimelikeOps, TimedeltaDelegateMixin):
    def __new__(cls, data: Optional[Any] = ..., unit: Optional[Any] = ..., freq: Optional[Any] = ..., closed: Optional[Any] = ..., dtype: Any = ..., copy: bool = ..., name: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def get_value_maybe_box(self, series: Any, key: Timedelta) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def is_type_compatible(self, typ: Any) -> bool: ...
    @property
    def inferred_type(self) -> str: ...
    def insert(self, loc: Any, item: Any) -> Any: ...

def timedelta_range(start: Any = ..., end: Any = ..., periods: Any = ..., freq: Any = ..., name: Any = ..., closed: Any = ...) -> TimedeltaIndex: ...



from pandas._libs.tslibs.timedeltas import Timedelta as Timedelta
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from typing import Any

def to_timedelta(arg: Any, unit: str = ..., errors: str = ...) -> Any: ...



import numpy as np
from pandas._libs.tslibs import  Timestamp as Timestamp, timezones as timezones, tzconversion as tzconversion
from pandas.core.algorithms import checked_add_with_arr as checked_add_with_arr
from pandas.core.arrays import datetimelike as dtl
from pandas.core.arrays._ranges import generate_regular_range as generate_regular_range

from pandas.core.dtypes.dtypes import DatetimeTZDtype as DatetimeTZDtype
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCPandasArray as ABCPandasArray, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.errors import PerformanceWarning as PerformanceWarning
from pandas.tseries.frequencies import get_period_alias as get_period_alias, to_offset as to_offset
from pandas.tseries.offsets import Day as Day, Tick as Tick
from typing import Any, Optional, Union

def tz_to_dtype(tz: Any) -> Any: ...

class DatetimeArray(dtl.DatetimeLikeArrayMixin, dtl.TimelikeOps, dtl.DatelikeOps):
    __array_priority__: int = ...
    def __init__(self, values: Any, dtype: Any = ..., freq: Optional[Any] = ..., copy: bool = ...) -> None: ...
    @property
    def dtype(self) -> Union[np.dtype, DatetimeTZDtype]: ...
    @property
    def tz(self) -> Any: ...
    @tz.setter
    def tz(self, value: Any) -> None: ...
    @property
    def tzinfo(self) -> Any: ...
    @property
    def is_normalized(self) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __iter__(self) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def tz_convert(self, tz: Any) -> Any: ...
    def tz_localize(self, tz: Any, ambiguous: str = ..., nonexistent: str = ...) -> Any: ...
    def to_pydatetime(self) -> Any: ...
    def normalize(self) -> Any: ...
    def to_period(self, freq: Optional[Any] = ...) -> Any: ...
    def to_perioddelta(self, freq: Any) -> Any: ...
    def month_name(self, locale: Optional[Any] = ...) -> Any: ...
    def day_name(self, locale: Optional[Any] = ...) -> Any: ...
    @property
    def time(self) -> Any: ...
    @property
    def timetz(self) -> Any: ...
    @property
    def date(self) -> Any: ...
    year: Any = ...
    month: Any = ...
    day: Any = ...
    hour: Any = ...
    minute: Any = ...
    second: Any = ...
    microsecond: Any = ...
    nanosecond: Any = ...
    weekofyear: Any = ...
    week: Any = ...
    dayofweek: Any = ...
    weekday: Any = ...
    dayofyear: Any = ...
    quarter: Any = ...
    days_in_month: Any = ...
    daysinmonth: Any = ...
    is_month_start: Any = ...
    is_month_end: Any = ...
    is_quarter_start: Any = ...
    is_quarter_end: Any = ...
    is_year_start: Any = ...
    is_year_end: Any = ...
    is_leap_year: Any = ...
    def to_julian_date(self) -> Any: ...

def sequence_to_dt64ns(data: Any, dtype: Optional[Any] = ..., copy: bool = ..., tz: Optional[Any] = ..., dayfirst: bool = ..., yearfirst: bool = ..., ambiguous: str = ...) -> Any: ...
def objects_to_datetime64ns(data: Any, dayfirst: Any, yearfirst: Any, utc: bool = ..., errors: str = ..., require_iso8601: bool = ..., allow_object: bool = ...) -> Any: ...
def maybe_convert_dtype(data: Any, copy: Any) -> Any: ...
def maybe_infer_tz(tz: Any, inferred_tz: Any) -> Any: ...
def validate_tz_from_dtype(dtype: Any, tz: Any) -> Any: ...



from pandas.core.accessor import register_dataframe_accessor as register_dataframe_accessor, register_index_accessor as register_index_accessor, register_series_accessor as register_series_accessor
from pandas.core.algorithms import take as take
from pandas.core.arrays import ExtensionArray as ExtensionArray, ExtensionScalarOpsMixin as ExtensionScalarOpsMixin
from pandas.core.dtypes.dtypes import ExtensionDtype as ExtensionDtype, register_extension_dtype as register_extension_dtype



from pandas import DataFrame as DataFrame
from pandas.core import ops as ops
from typing import Any

def pytest_addoption(parser: Any) -> None: ...
def pytest_runtest_setup(item: Any) -> None: ...
def configure_tests() -> None: ...
def add_imports(doctest_namespace: Any) -> None: ...
def spmatrix(request: Any) -> Any: ...
def axis(request: Any) -> Any: ...
axis_frame = axis

def axis_series(request: Any) -> Any: ...
def ip() -> Any: ...
def observed(request: Any) -> Any: ...
def ordered_fixture(request: Any) -> Any: ...
def all_arithmetic_operators(request: Any) -> Any: ...
def all_arithmetic_functions(request: Any) -> Any: ...
def all_numeric_reductions(request: Any) -> Any: ...
def all_boolean_reductions(request: Any) -> Any: ...
def cython_table_items(request: Any) -> Any: ...
def all_compare_operators(request: Any) -> Any: ...
def compare_operators_no_eq_ne(request: Any) -> Any: ...
def all_logical_operators(request: Any) -> Any: ...
def compression(request: Any) -> Any: ...
def compression_only(request: Any) -> Any: ...
def writable(request: Any) -> Any: ...
def datetime_tz_utc() -> Any: ...
def utc_fixture(request: Any) -> Any: ...
def join_type(request: Any) -> Any: ...
def strict_data_files(pytestconfig: Any) -> Any: ...
def datapath(strict_data_files: Any) -> Any: ...
def iris(datapath: Any) -> Any: ...
def nselect_method(request: Any) -> Any: ...
def closed(request: Any) -> Any: ...
def other_closed(request: Any) -> Any: ...
def nulls_fixture(request: Any) -> Any: ...
nulls_fixture2 = nulls_fixture

def unique_nulls_fixture(request: Any) -> Any: ...
unique_nulls_fixture2 = unique_nulls_fixture
TIMEZONES: Any
TIMEZONE_IDS: Any

def tz_naive_fixture(request: Any) -> Any: ...
def tz_aware_fixture(request: Any) -> Any: ...
tz_aware_fixture2 = tz_aware_fixture
UNSIGNED_INT_DTYPES: Any
UNSIGNED_EA_INT_DTYPES: Any
SIGNED_INT_DTYPES: Any
SIGNED_EA_INT_DTYPES: Any
ALL_INT_DTYPES: Any
ALL_EA_INT_DTYPES: Any
FLOAT_DTYPES: Any
COMPLEX_DTYPES: Any
STRING_DTYPES: Any
DATETIME64_DTYPES: Any
TIMEDELTA64_DTYPES: Any
BOOL_DTYPES: Any
BYTES_DTYPES: Any
OBJECT_DTYPES: Any
ALL_REAL_DTYPES: Any
ALL_NUMPY_DTYPES: Any

def string_dtype(request: Any) -> Any: ...
def bytes_dtype(request: Any) -> Any: ...
def object_dtype(request: Any) -> Any: ...
def datetime64_dtype(request: Any) -> Any: ...
def timedelta64_dtype(request: Any) -> Any: ...
def float_dtype(request: Any) -> Any: ...
def complex_dtype(request: Any) -> Any: ...
def sint_dtype(request: Any) -> Any: ...
def uint_dtype(request: Any) -> Any: ...
def any_int_dtype(request: Any) -> Any: ...
def any_nullable_int_dtype(request: Any) -> Any: ...
def any_real_dtype(request: Any) -> Any: ...
def any_numpy_dtype(request: Any) -> Any: ...

ids: Any

def any_skipna_inferred_dtype(request: Any) -> Any: ...
def tick_classes(request: Any) -> Any: ...

cls: Any

def float_frame() -> Any: ...
def index_or_series(request: Any) -> Any: ...
def dict_subclass() -> Any: ...
def non_mapping_dict_subclass() -> Any: ...



from pandas import DataFrame as DataFrame, get_option as get_option
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.io.common import get_filepath_or_buffer as get_filepath_or_buffer, is_gcs_url as is_gcs_url, is_s3_url as is_s3_url
from typing import Any, Optional

def get_engine(engine: str) -> BaseImpl: ...

class BaseImpl:
    @staticmethod
    def validate_dataframe(df: DataFrame) -> Any: ...
    def write(self, df: DataFrame, path: Any, compression: Any, **kwargs: Any) -> Any: ...
    def read(self, path: Any, columns: Optional[Any] = ..., **kwargs: Any) -> None: ...

class PyArrowImpl(BaseImpl):
    api: Any = ...
    def __init__(self) -> None: ...
    def write(self, df: DataFrame, path: Any, compression: Any = ..., coerce_timestamps: Any = ..., index: Optional[bool]=..., partition_cols: Any = ..., **kwargs: Any) -> Any: ...
    def read(self, path: Any, columns: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class FastParquetImpl(BaseImpl):
    api: Any = ...
    def __init__(self) -> None: ...
    def write(self, df: DataFrame, path: Any, compression: Any = ..., index: Any = ..., partition_cols: Any = ..., **kwargs: Any) -> Any: ...
    def read(self, path: Any, columns: Optional[Any] = ..., **kwargs: Any) -> Any: ...

def to_parquet(df: DataFrame, path: Any, engine: str=..., compression: Any = ..., index: Optional[bool]=..., partition_cols: Any = ..., **kwargs: Any) -> Any: ...
def read_parquet(path: Any, engine: str=..., columns: Any = ..., **kwargs: Any) -> Any: ...



import numpy as np
from pandas import Series as Series
from pandas.core.construction import array as array, extract_array as extract_array
from pandas.core.dtypes.cast import construct_1d_object_array_from_listlike as construct_1d_object_array_from_listlike, infer_dtype_from_array as infer_dtype_from_array, maybe_promote as maybe_promote
from pandas.core.dtypes.common import ensure_float64 as ensure_float64, ensure_int64 as ensure_int64, ensure_object as ensure_object, ensure_platform_int as ensure_platform_int, ensure_uint64 as ensure_uint64, is_array_like as is_array_like, is_bool_dtype as is_bool_dtype, is_categorical_dtype as is_categorical_dtype, is_complex_dtype as is_complex_dtype, is_datetime64_any_dtype as is_datetime64_any_dtype, is_datetime64_dtype as is_datetime64_dtype, is_datetime64_ns_dtype as is_datetime64_ns_dtype, is_extension_array_dtype as is_extension_array_dtype, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_list_like as is_list_like, is_numeric_dtype as is_numeric_dtype, is_object_dtype as is_object_dtype, is_period_dtype as is_period_dtype, is_scalar as is_scalar, is_signed_integer_dtype as is_signed_integer_dtype, is_timedelta64_dtype as is_timedelta64_dtype, is_unsigned_integer_dtype as is_unsigned_integer_dtype, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.generic import ABCIndex as ABCIndex, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna, na_value_for_dtype as na_value_for_dtype
from pandas.core.indexers import validate_indices as validate_indices
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Optional, Tuple, Union

def unique(values: Any) -> Any: ...
unique1d = unique

def isin(comps: Any, values: Any) -> np.ndarray: ...
def factorize(values: Any, sort: bool=..., na_sentinel: int=..., size_hint: Optional[int]=...) -> Tuple[np.ndarray, Union[np.ndarray, ABCIndex]]: ...
def value_counts(values: Any, sort: bool=..., ascending: bool=..., normalize: bool=..., bins: Any = ..., dropna: bool=...) -> Series: ...
def duplicated(values: Any, keep: Any = ...) -> np.ndarray: ...
def mode(values: Any, dropna: bool=...) -> Series: ...
def rank(values: Any, axis: int=..., method: str=..., na_option: str=..., ascending: bool=..., pct: bool=...) -> Any: ...
def checked_add_with_arr(arr: Any, b: Any, arr_mask: Optional[Any] = ..., b_mask: Optional[Any] = ...) -> Any: ...
def quantile(x: Any, q: Any, interpolation_method: str = ...) -> Any: ...

class SelectN:
    obj: Any = ...
    n: Any = ...
    keep: Any = ...
    def __init__(self, obj: Any, n: int, keep: str) -> None: ...
    def nlargest(self) -> Any: ...
    def nsmallest(self) -> Any: ...
    @staticmethod
    def is_valid_dtype_n_method(dtype: Any) -> bool: ...

class SelectNSeries(SelectN):
    def compute(self, method: Any) -> Any: ...

class SelectNFrame(SelectN):
    columns: Any = ...
    def __init__(self, obj: Any, n: int, keep: str, columns: Any) -> None: ...
    def compute(self, method: Any) -> Any: ...

def take(arr: Any, indices: Any, axis: int=..., allow_fill: bool=..., fill_value: Any = ...) -> Any: ...
def take_nd(arr: Any, indexer: Any, axis: int=..., out: Any = ..., fill_value: Any = ..., allow_fill: bool=...) -> Any: ...
take_1d = take_nd

def take_2d_multi(arr: Any, indexer: Any, fill_value: Any = ...) -> Any: ...
def searchsorted(arr: Any, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
def diff(arr: Any, n: int, axis: int=..., stacklevel: Any = ...) -> Any: ...
def safe_sort(values: Any, codes: Any = ..., na_sentinel: int=..., assume_unique: bool=..., verify: bool=...) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]: ...



from pandas._libs import Timestamp as Timestamp
from pandas.core.base import PandasObject as PandasObject
from pandas.core.dtypes.cast import find_common_type as find_common_type, infer_dtype_from_scalar as infer_dtype_from_scalar, maybe_convert_objects as maybe_convert_objects, maybe_promote as maybe_promote

from pandas.core.dtypes.dtypes import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexers import maybe_convert_indices as maybe_convert_indices
from pandas.core.indexes.api import Index as Index, MultiIndex as MultiIndex, ensure_index as ensure_index
from pandas.core.internals.blocks import Block as Block, CategoricalBlock as CategoricalBlock, DatetimeTZBlock as DatetimeTZBlock, ExtensionBlock as ExtensionBlock, ObjectValuesExtensionBlock as ObjectValuesExtensionBlock, get_block_type as get_block_type, make_block as make_block
from pandas.core.internals.concat import combine_concat_plans as combine_concat_plans, concatenate_join_units as concatenate_join_units, get_mgr_concatenation_plan as get_mgr_concatenation_plan, is_uniform_join_units as is_uniform_join_units
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.util._validators import validate_bool_kwarg as validate_bool_kwarg
from typing import Any, List, Optional, Sequence, Union

class BlockManager(PandasObject):
    axes: Any = ...
    blocks: Any = ...
    def __init__(self, blocks: Sequence[Block], axes: Sequence[Index], do_integrity_check: bool=...) -> None: ...
    def make_empty(self, axes: Optional[Any] = ...) -> Any: ...
    def __nonzero__(self) -> Any: ...
    __bool__: Any = ...
    @property
    def shape(self) -> Any: ...
    @property
    def ndim(self) -> int: ...
    def set_axis(self, axis: Any, new_labels: Any) -> None: ...
    def rename_axis(self, mapper: Any, axis: Any, copy: bool = ..., level: Optional[Any] = ...) -> Any: ...
    @property
    def items(self) -> Any: ...
    def get_dtype_counts(self) -> Any: ...
    def get_dtypes(self) -> Any: ...
    def __len__(self) -> int: ...
    def reduce(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def apply(self, f: Any, filter: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def quantile(self, axis: int = ..., consolidate: bool = ..., transposed: bool = ..., interpolation: str = ..., qs: Optional[Any] = ..., numeric_only: Optional[Any] = ...) -> Any: ...
    def isna(self, func: Any) -> Any: ...
    def where(self, **kwargs: Any) -> Any: ...
    def setitem(self, **kwargs: Any) -> Any: ...
    def putmask(self, **kwargs: Any) -> Any: ...
    def diff(self, **kwargs: Any) -> Any: ...
    def interpolate(self, **kwargs: Any) -> Any: ...
    def shift(self, **kwargs: Any) -> Any: ...
    def fillna(self, **kwargs: Any) -> Any: ...
    def downcast(self, **kwargs: Any) -> Any: ...
    def astype(self, dtype: Any, copy: bool=..., errors: str=...) -> Any: ...
    def convert(self, **kwargs: Any) -> Any: ...
    def replace(self, value: Any, **kwargs: Any) -> Any: ...
    def replace_list(self, src_list: Any, dest_list: Any, inplace: bool = ..., regex: bool = ...) -> Any: ...
    def is_consolidated(self) -> Any: ...
    @property
    def is_mixed_type(self) -> Any: ...
    @property
    def is_numeric_mixed_type(self) -> Any: ...
    @property
    def is_datelike_mixed_type(self) -> Any: ...
    @property
    def any_extension_types(self) -> Any: ...
    @property
    def is_view(self) -> Any: ...
    def get_bool_data(self, copy: bool = ...) -> Any: ...
    def get_numeric_data(self, copy: bool = ...) -> Any: ...
    def combine(self, blocks: Any, copy: bool = ...) -> Any: ...
    def get_slice(self, slobj: slice, axis: int=...) -> Any: ...
    def __contains__(self, item: Any) -> bool: ...
    @property
    def nblocks(self) -> int: ...
    def copy(self, deep: bool = ...) -> Any: ...
    def as_array(self, transpose: bool = ..., items: Optional[Any] = ...) -> Any: ...
    def to_dict(self, copy: bool = ...) -> Any: ...
    def fast_xs(self, loc: Any) -> Any: ...
    def consolidate(self) -> Any: ...
    def get(self, item: Any) -> Any: ...
    def iget(self, i: Any) -> Any: ...
    def delete(self, item: Any) -> None: ...
    def set(self, item: Any, value: Any) -> Any: ...
    def insert(self, loc: int, item: Any, value: Any, allow_duplicates: bool=...) -> Any: ...
    def reindex_axis(self, new_index: Any, axis: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., fill_value: Optional[Any] = ..., copy: bool = ...) -> Any: ...
    def reindex_indexer(self, new_axis: Any, indexer: Any, axis: Any, fill_value: Optional[Any] = ..., allow_dups: bool = ..., copy: bool = ...) -> Any: ...
    def take(self, indexer: Any, axis: int = ..., verify: bool = ..., convert: bool = ...) -> Any: ...
    def equals(self, other: Any) -> Any: ...
    def unstack(self, unstacker_func: Any, fill_value: Any) -> Any: ...

class SingleBlockManager(BlockManager):
    ndim: int = ...
    axes: Any = ...
    blocks: Any = ...
    def __init__(self, block: Block, axis: Union[Index, List[Index]], do_integrity_check: bool=..., fastpath: bool=...) -> None: ...
    def get_slice(self, slobj: Any, axis: int = ...) -> Any: ...
    @property
    def index(self) -> Any: ...
    @property
    def dtype(self) -> Any: ...
    @property
    def array_dtype(self) -> Any: ...
    def get_dtype_counts(self) -> Any: ...
    def get_dtypes(self) -> Any: ...
    def external_values(self) -> Any: ...
    def internal_values(self) -> Any: ...
    def get_values(self) -> Any: ...
    def is_consolidated(self) -> Any: ...
    def delete(self, item: Any) -> None: ...
    def fast_xs(self, loc: Any) -> Any: ...
    def concat(self, to_concat: Any, new_axis: Any) -> Any: ...

def create_block_manager_from_blocks(blocks: Any, axes: Any) -> Any: ...
def create_block_manager_from_arrays(arrays: Any, names: Any, axes: Any) -> Any: ...
def construction_error(tot_items: Any, block_shape: Any, axes: Any, e: Optional[Any] = ...) -> None: ...
def form_blocks(arrays: Any, names: Any, axes: Any) -> Any: ...
def concatenate_block_managers(mgrs_indexers: Any, axes: Any, concat_axis: Any, copy: Any) -> Any: ...

def _transform_index(index: Any, func: Any, level: Any) -> Any: ...



from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from typing import Any, IO, Optional, Tuple, AnyStr

s3fs: Any

def get_file_and_filesystem(filepath_or_buffer: FilePathOrBuffer[AnyStr], mode: Optional[str]=...) -> Tuple[IO[Any], Any]: ...
def get_filepath_or_buffer(filepath_or_buffer: FilePathOrBuffer[AnyStr], encoding: Optional[str]=..., compression: Optional[str]=..., mode: Optional[str]=...) -> Tuple[IO[Any], Optional[str], Optional[str], bool]: ...



import ast
from pandas.core.computation.ops import BinOp as BinOp, Constant as Constant, Div as Div, FuncNode as FuncNode, Op as Op, Term as Term, UnaryOp as UnaryOp, UndefinedVariableError as UndefinedVariableError, is_term as is_term
from pandas.core.computation.parsing import clean_backtick_quoted_toks as clean_backtick_quoted_toks, tokenize_string as tokenize_string
from pandas.core.computation.scope import Scope as Scope
from typing import Any, Optional, Type

intersection: Any

def disallow(nodes: Any) -> Any: ...
def add_ops(op_classes: Any) -> Any: ...

class BaseExprVisitor(ast.NodeVisitor):
    const_type: Type[Term] = ...
    term_type: Any = ...
    binary_ops: Any = ...
    binary_op_nodes: Any = ...
    binary_op_nodes_map: Any = ...
    unary_ops: Any = ...
    unary_op_nodes: Any = ...
    unary_op_nodes_map: Any = ...
    rewrite_map: Any = ...
    env: Any = ...
    engine: Any = ...
    parser: Any = ...
    preparser: Any = ...
    assigner: Any = ...
    def __init__(self, env: Any, engine: Any, parser: Any, preparser: Any = ...) -> None: ...
    def visit(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Module(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Expr(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_BinOp(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Div(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_UnaryOp(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Name(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_NameConstant(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Num(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Constant(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Str(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_List(self, node: Any, **kwargs: Any) -> Any: ...
    visit_Tuple: Any = ...
    def visit_Index(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Subscript(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Slice(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Assign(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Attribute(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Call(self, node: Any, side: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def translate_In(self, op: Any) -> Any: ...
    def visit_Compare(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_BoolOp(self, node: Any, **kwargs: Any) -> Any: ...

class PandasExprVisitor(BaseExprVisitor):
    def __init__(self, env: Any, engine: Any, parser: Any, preparser: Any = ...) -> None: ...

class PythonExprVisitor(BaseExprVisitor):
    def __init__(self, env: Any, engine: Any, parser: Any, preparser: Any = ...) -> None: ...

class Expr:
    env: Scope
    engine: str
    parser: str
    expr: Any = ...
    terms: Any = ...
    def __init__(self, expr: Any, engine: str=..., parser: str=..., env: Optional[Scope]=..., level: int=...) -> None: ...
    @property
    def assigner(self) -> Any: ...
    def __call__(self) -> Any: ...
    def __len__(self) -> int: ...
    def parse(self) -> Any: ...
    @property
    def names(self) -> Any: ...



from typing import Any, Optional
from pandas.core.dtypes.missing import isna as isna

def mask_missing(arr: Any, values_to_mask: Any) -> Any: ...
def clean_fill_method(method: Any, allow_nearest: bool = ...) -> Any: ...
def clean_interp_method(method: Any, **kwargs: Any) -> Any: ...
def find_valid_index(values: Any, how: str) -> Any: ...
def interpolate_1d(xvalues: Any, yvalues: Any, method: str = ..., limit: Optional[Any] = ..., limit_direction: str = ..., limit_area: Optional[Any] = ..., fill_value: Optional[Any] = ..., bounds_error: bool = ..., order: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def interpolate_2d(values: Any, method: str = ..., axis: int = ..., limit: Optional[Any] = ..., fill_value: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
def pad_1d(values: Any, limit: Optional[Any] = ..., mask: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
def backfill_1d(values: Any, limit: Optional[Any] = ..., mask: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
def pad_2d(values: Any, limit: Optional[Any] = ..., mask: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
def backfill_2d(values: Any, limit: Optional[Any] = ..., mask: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
def get_fill_func(method: Any) -> Any: ...
def clean_reindex_fill_method(method: Any) -> Any: ...



import numpy as np
from pandas._libs.tslibs import Timestamp as Timestamp
from pandas.core.computation.common import result_type_many as result_type_many

from pandas.io.formats.printing import pprint_thing as pprint_thing, pprint_thing_encoded as pprint_thing_encoded
from typing import Any, Optional

class UndefinedVariableError(NameError):
    def __init__(self, name: Any, is_local: bool) -> None: ...

class Term:
    def __new__(cls, name: Any, env: Any, side: Optional[Any] = ..., encoding: Optional[Any] = ...) -> Any: ...
    is_local: bool
    env: Any = ...
    side: Any = ...
    encoding: Any = ...
    def __init__(self, name: Any, env: Any, side: Optional[Any] = ..., encoding: Optional[Any] = ...) -> None: ...
    @property
    def local_name(self) -> str: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def evaluate(self, *args: Any, **kwargs: Any) -> Any: ...
    def update(self, value: Any) -> None: ...
    @property
    def is_scalar(self) -> bool: ...
    @property
    def type(self) -> Any: ...
    return_type: Any = ...
    @property
    def raw(self) -> str: ...
    @property
    def is_datetime(self) -> bool: ...
    @property
    def value(self) -> Any: ...
    @value.setter
    def value(self, new_value: Any) -> None: ...
    @property
    def name(self) -> Any: ...
    @property
    def ndim(self) -> int: ...

class Constant(Term):
    def __init__(self, value: Any, env: Any, side: Optional[Any] = ..., encoding: Optional[Any] = ...) -> None: ...
    @property
    def name(self) -> Any: ...

class Op:
    op: str
    operands: Any = ...
    encoding: Any = ...
    def __init__(self, op: str, operands: Any, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Any: ...
    @property
    def return_type(self) -> Any: ...
    @property
    def has_invalid_return_type(self) -> bool: ...
    @property
    def operand_types(self) -> Any: ...
    @property
    def is_scalar(self) -> bool: ...
    @property
    def is_datetime(self) -> bool: ...

def is_term(obj: Any) -> bool: ...

class BinOp(Op):
    lhs: Any = ...
    rhs: Any = ...
    func: Any = ...
    def __init__(self, op: str, lhs: Any, rhs: Any, **kwargs: Any) -> None: ...
    def __call__(self, env: Any) -> Any: ...
    def evaluate(self, env: Any, engine: str, parser: Any, term_type: Any, eval_in_python: Any) -> Any: ...
    def convert_values(self) -> Any: ...

def isnumeric(dtype: Any) -> bool: ...

class Div(BinOp):
    def __init__(self, lhs: Any, rhs: Any, **kwargs: Any) -> None: ...

class UnaryOp(Op):
    operand: Any = ...
    func: Any = ...
    def __init__(self, op: str, operand: Any) -> None: ...
    def __call__(self, env: Any) -> Any: ...
    @property
    def return_type(self) -> np.dtype: ...

class MathCall(Op):
    func: Any = ...
    def __init__(self, func: Any, args: Any) -> None: ...
    def __call__(self, env: Any) -> Any: ...

class FuncNode:
    name: Any = ...
    func: Any = ...
    def __init__(self, name: str) -> None: ...
    def __call__(self, *args: Any) -> Any: ...



from datetime import timedelta
from typing import Any, Union
import numpy as np

class Timedelta:

    def __init__(self, value: Union[Timedelta, timedelta, np.timedelta64, str, int], unit: str = ..., **kwargs: Any) -> None: ...


import numpy as np
from pandas._libs.sparse import BlockIndex as BlockIndex, IntIndex as IntIndex, SparseIndex as SparseIndex
from pandas.core.arrays import ExtensionArray as ExtensionArray, ExtensionOpsMixin as ExtensionOpsMixin
from pandas.core.arrays.sparse.dtype import SparseDtype as SparseDtype
from pandas.core.base import PandasObject as PandasObject
from pandas.core.construction import sanitize_array as sanitize_array
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe, construct_1d_arraylike_from_scalar as construct_1d_arraylike_from_scalar, find_common_type as find_common_type, infer_dtype_from_scalar as infer_dtype_from_scalar
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries, ABCSparseArray as ABCSparseArray
from pandas.core.dtypes.missing import isna as isna, na_value_for_dtype as na_value_for_dtype, notna as notna
from pandas.core.missing import interpolate_2d as interpolate_2d
from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.errors import PerformanceWarning as PerformanceWarning
from typing import Any, Optional

class SparseArray(PandasObject, ExtensionArray, ExtensionOpsMixin):
    def __init__(self, data: Any, sparse_index: Optional[Any] = ..., index: Optional[Any] = ..., fill_value: Optional[Any] = ..., kind: str = ..., dtype: Optional[Any] = ..., copy: bool = ...) -> None: ...
    @classmethod
    def from_spmatrix(cls, data: Any) -> Any: ...
    def __array__(self, dtype: Any = ..., copy: Any = ...) -> np.ndarray: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    @property
    def sp_index(self) -> Any: ...
    @property
    def sp_values(self) -> Any: ...
    @property
    def dtype(self) -> Any: ...
    @property
    def fill_value(self) -> Any: ...
    @fill_value.setter
    def fill_value(self, value: Any) -> None: ...
    @property
    def kind(self) -> str: ...
    def __len__(self) -> int: ...
    @property
    def nbytes(self) -> int: ...
    @property
    def density(self) -> Any: ...
    @property
    def npoints(self) -> int: ...
    def isna(self) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    def shift(self, periods: int = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def unique(self) -> Any: ...
    def factorize(self, na_sentinel: int = ...) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...
    def __getitem__(self, key: Any) -> Any: ...
    def take(self, indices: Any, allow_fill: bool = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def searchsorted(self, v: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def copy(self) -> Any: ...
    def astype(self, dtype: Optional[Any] = ..., copy: bool = ...) -> Any: ...
    def map(self, mapper: Any) -> Any: ...
    def to_dense(self) -> Any: ...
    def nonzero(self) -> Any: ...
    def all(self, axis: Optional[Any] = ..., *args: Any, **kwargs: Any) -> Any: ...
    def any(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def sum(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def cumsum(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def transpose(self, *axes: Any) -> Any: ...
    @property
    def T(self) -> Any: ...
    def __array_ufunc__(self, ufunc: Any, method: Any, *inputs: Any, **kwargs: Any) -> Any: ...
    def __abs__(self) -> Any: ...

def make_sparse(arr: Any, kind: str = ..., fill_value: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ...) -> Any: ...



from __future__ import annotations

import sys

import numpy.ma as np

from pandas import datetime
from pandas._typing import Axes, Axis, Dtype as Dtype, FilePathOrBuffer, Level, Renamer, Column, Label, FrameOrSeries, \
    ArrayLike, AnyArrayLike, GoogleCredentials, Scalar, ReplaceMethod, ToReplace, ReplaceValue, Frequency, AxisOption, \
    Orientation, Function, AggregationFunction, GroupByObject, GeneralDuplicatesKeepStrategy, InterpolationMethod, \
    CorrelationMethod, SortKind, JoinType, FillMethod, ErrorsStrategy, NaSortPosition, FillValue, TimestampMethod, \
    ValueKeyFunc, IndexKeyFunc
from pandas.core.accessor import CachedAccessor
from pandas.core.base import PandasObject
from pandas.core.generic import NDFrame
from pandas.core.groupby import generic as groupby_generic
from pandas.core.groupby.grouper import Grouper
from pandas.core.indexes.api import Index
from pandas.core.series import Series
from pandas.io.formats import format as fmt
from pandas.io.formats.format import formatters_type, VALID_JUSTIFY_PARAMETERS, FloatFormatType
from pandas.io.formats.style import Styler
from typing import Any, Hashable, IO, Iterable, List, Optional, Sequence, Tuple, Union, Dict, Mapping, Type, \
    overload, Iterator, Callable, AnyStr

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

ExportOrientation = Literal['dict', 'list', 'series', 'split', 'records', 'index']
CompressionType = Literal['snappy', 'gzip', 'brotli']
IfExistStrategy = Literal['fail', 'replace', 'append']
ParquetEngine = Literal['auto', 'pyarrow', 'fastparquet']
DropTypes = Literal['any', 'all']
KeepStrategy = Literal['first', 'last', 'all']
UpdateJoinType = Literal['left']
UpdateErrorsStrategy = Literal['raise', 'ignore']
ApplyResultType = Literal['expand', 'reduce', 'broadcast']
MergeType = JoinType
MergeValidationMethod = Literal["one_to_one", "1:1", "one_to_many", "1:m", "many_to_one", "m:1", "many_to_many", "m:m"]

IndexArray = Union[Series, Index, np.ndarray, Iterator]
CoercibleIntoDataFrame = Union[Dict[str, Scalar], Dict[str, Series], Dict[str, Tuple[Scalar, ...]], Dict[str, Iterable[Scalar]]]

ColumnWidth = Union[str, int]
ColumnSpace = Union[ColumnWidth, Sequence[ColumnWidth], Dict[Column, ColumnWidth]]

TransformFunction = AggregationFunction

class DataFrame(NDFrame):
    plot: CachedAccessor = ...
    hist: Callable[..., Any] = ...
    boxplot: Callable[..., Any] = ...
    sparse: CachedAccessor = ...
    def __init__(self, data: Any = ..., index: Optional[Axes[Any]] = ..., columns: Optional[Axes[Any]] = ..., dtype: Optional[Dtype] = ..., copy: bool = ...) -> None: ...
    def __len__(self) -> int: ...
    def __le__(self, other: Scalar) -> DataFrame: ...
    def __lt__(self, other: Scalar) -> DataFrame: ...
    def __ge__(self, other: Scalar) -> DataFrame: ...
    def __gt__(self, other: Scalar) -> DataFrame: ...
    def __mul__(self, other: Scalar) -> DataFrame: ...
    def __truediv__(self, other: Union[Scalar, DataFrame]) -> DataFrame: ...
    def __floordiv__(self, other: Union[Scalar, DataFrame]) -> DataFrame: ...
    def __mod__(self, other: Union[Scalar, DataFrame]) -> DataFrame: ...
    def __divmod__(self, other: Union[Scalar, DataFrame]) -> Tuple[DataFrame, DataFrame]: ...
    def __rdivmod__(self, other: Union[Scalar, DataFrame]) -> Tuple[DataFrame, DataFrame]: ...
    def __and__(self, other: DataFrame) -> DataFrame: ...
    def __or__(self, other: DataFrame) -> DataFrame: ...
    def __add__(self, other: Union[Scalar, DataFrame]) -> DataFrame: ...
    def __sub__(self, other: Union[Scalar, DataFrame]) -> DataFrame: ...
    def __matmul__(self, other: Union[FrameOrSeries, ArrayLike]) -> FrameOrSeries: ...
    def __rmatmul__(self, other: Union[FrameOrSeries, ArrayLike]) -> FrameOrSeries: ...
    # Type ignoring the first declaration because for some reason mypy thinks that np.ndarray overlaps Column
    @overload
    def __getitem__(self, key: Column) -> Series: ...  # type: ignore
    @overload
    def __getitem__(self, key: Union[Series, Index, DataFrame, List[Column], slice, np.ndarray]) -> DataFrame: ...
    @overload
    def __setitem__(self, key: Column, value: Any) -> DataFrame: ...
    @overload
    def __setitem__(self, key: Union[Series, Index, DataFrame, List[Column], slice, np.ndarray], value: Any) -> DataFrame: ...
    @property
    def axes(self) -> List[Index]: ...
    @property
    def shape(self) -> Tuple[int, int]: ...
    def to_string(self, buf: Optional[FilePathOrBuffer[str]] = ..., columns: Optional[Sequence[str]] = ..., col_space: Optional[Union[int, Sequence[int], Dict[str, int]]] = ..., header: Union[bool, Sequence[str]] = ..., index: bool = ..., na_rep: str = ..., formatters: Optional[fmt.formatters_type] = ..., float_format: Optional[fmt.float_format_type] = ..., sparsify: Optional[bool] = ..., index_names: bool = ..., justify: Optional[str] = ..., max_rows: Optional[int] = ..., min_rows: Optional[int] = ..., max_cols: Optional[int] = ..., show_dimensions: bool = ..., decimal: str = ..., line_width: Optional[int] = ..., max_colwidth: Optional[int] = ..., encoding: Optional[str] = ...) -> Optional[str]: ...
    @property
    def style(self) -> Styler: ...
    def items(self) -> Iterable[Tuple[Label, Series]]: ...
    def iteritems(self) -> Iterable[Tuple[Label, Series]]: ...
    def iterrows(self) -> Iterable[Tuple[Label, Series]]: ...
    # This isn't exact, first argument could(!) be an Index, the rest column values
    def itertuples(self, index: bool = ..., name: str = ...) -> Iterable[Tuple[Any, ...]]: ...
    def dot(self, other: Union[FrameOrSeries, ArrayLike]) -> FrameOrSeries: ...
    @classmethod
    def from_dict(cls: Any, data: Dict[Union[Label, Column], Union[Union[AnyArrayLike, Iterable[Any]], Dict[Union[Label, Column], Any]]], orient: Orientation = ..., dtype: Optional[Dtype] = ..., columns: Optional[Sequence[str]] = ...) -> DataFrame: ...
    def to_numpy(self, dtype: Union[str, np.dtype] = ..., copy: bool = ..., na_value: Optional[Any] = ...) -> np.ndarray: ...
    def to_dict(self, orient: ExportOrientation = ..., into: Type[Mapping[Column, Any]] = ...) -> Union[Mapping[Column, Any], List[Any]]: ...
    def to_gbq(self, destination_table: str, project_id: Optional[str] = ..., chunksize: Optional[int] = ..., reauth: bool = ..., if_exists: IfExistStrategy = ..., auth_local_webserver: bool = ..., table_schema: Optional[List[Dict[str, Any]]] = ..., location: Optional[str] = ..., progress_bar: bool = ..., credentials: Optional[GoogleCredentials] = ...) -> None: ...
    @classmethod
    def from_records(cls: Any, data: Union[np.ndarray, List[Tuple[Any, ...]], Dict[Any, Any], DataFrame], index: Union[Sequence[str], ArrayLike] = ..., exclude: Sequence[Column] = ..., columns: Sequence[Column] = ..., coerce_float: bool = ..., nrows: Optional[int] = ...) -> DataFrame: ...
    def to_records(self, index: bool = ..., column_dtypes: Optional[Union[str, type, Dict[Column, Dtype]]] = ..., index_dtypes: Optional[Union[str, type, Dict[Column, Dtype]]] = ...) -> np.recarray: ...
    def to_stata(self, path: FilePathOrBuffer[AnyStr], convert_dates: Optional[Dict[Label, str]] = ..., write_index: bool = ..., byteorder: Optional[str] = ..., time_stamp: Optional[datetime.datetime] = ..., data_label: Optional[str] = ..., variable_labels: Optional[Dict[Label, str]] = ..., version: int = ..., convert_strl: Optional[Sequence[Label]] = ...) -> None: ...
    def to_feather(self, path: str, **kwargs: Any) -> None: ...
    def to_markdown(self, buf: Optional[IO[str]] = ..., mode: Optional[str] = ..., index: bool = ..., **kwargs: Any) -> Optional[str]: ...
    def to_parquet(self, path: Optional[FilePathOrBuffer[AnyStr]] = ..., engine: ParquetEngine = ..., compression: Optional[CompressionType] = ..., index: Optional[bool] = ..., partition_cols: Optional[List[Column]] = ..., **kwargs: Any) -> Optional[bytes]: ...
    def to_html(self, buf: Optional[Any] = ..., columns: Optional[Sequence[str]] = ..., col_space: ColumnSpace = ..., header: Union[bool, Sequence[str]] = ..., index: bool = ..., na_rep: str = ..., formatters: Optional[formatters_type] = ..., float_format: Optional[FloatFormatType] = ..., sparsify: Optional[bool] = ..., index_names: bool = ..., justify: Optional[VALID_JUSTIFY_PARAMETERS] = ..., max_rows: Optional[int] = ..., max_cols: Optional[int] = ..., show_dimensions: Union[bool, str] = ..., decimal: str = ..., bold_rows: bool = ..., classes: Optional[Sequence[str]] = ..., escape: bool = ..., notebook: bool = ..., border: Optional[int] = ..., table_id: Optional[str] = ..., render_links: bool = ..., encoding: Optional[str] = ...) -> str: ...
    def info(self, verbose: Optional[bool] = ..., buf: Optional[IO[str]] = ..., max_cols: Optional[int] = ..., memory_usage: Optional[Union[bool, str]] = ..., null_counts: Optional[bool] = ...) -> None: ...
    def memory_usage(self, index: Optional[bool] = ..., deep: Optional[bool] = ...) -> Series: ...
    def transpose(self, *args: Any, copy: bool = ...) -> DataFrame: ...
    @property
    def T(self) -> DataFrame: ...
    @overload
    def query(self, expr: str, inplace: Literal[False] = ..., **kwargs: Any) -> DataFrame: ...
    @overload
    def query(self, expr: str, inplace: Literal[True], **kwargs: Any) -> None: ...
    @overload
    def eval(self, expr: str, inplace: Literal[False] = ..., **kwargs: Any) -> Union[np.ndarray, int, float, PandasObject]: ...
    @overload
    def eval(self, expr: str, inplace: Literal[True], **kwargs: Any) -> None: ...
    def select_dtypes(self, include: Optional[Sequence[Union[str, Dtype]]] = ..., exclude: Optional[Sequence[Union[str, Dtype]]] = ...) -> DataFrame: ...
    def insert(self, loc: int, column: Union[Column, Hashable], value: Union[int, Series, ArrayLike], allow_duplicates: Optional[bool] = ...) -> None: ...
    def assign(self, **kwargs: Any) -> DataFrame: ...
    def lookup(self, row_labels: Sequence[Any], col_labels: Sequence[Column]) -> np.ndarray: ...
    def align(self, other: FrameOrSeries, join: JoinType = ..., axis: AxisOption = ..., level: Level = ..., copy: bool = ..., fill_value: Scalar = ..., method: Optional[FillMethod] = ..., limit: Optional[int] = ..., fill_axis: AxisOption = ..., broadcast_axis: AxisOption = ...) -> DataFrame: ...
    def reindex(self, *args: Any, **kwargs: Any) -> DataFrame: ...
    @overload
    def drop(self, labels: Optional[Sequence[Label]] = ..., axis: AxisOption = ..., index: Optional[Sequence[Label]] = ..., columns: Optional[Sequence[Label]] = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def drop(self, labels: Optional[Sequence[Label]] = ..., axis: AxisOption = ..., index: Optional[Sequence[Label]] = ..., columns: Optional[Sequence[Label]] = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def rename(self, mapper: Optional[Renamer] = ..., *, index: Optional[Renamer] = ..., columns: Optional[Renamer] = ..., axis: Optional[Axis] = ..., copy: bool = ..., inplace: Literal[False] = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ...) -> DataFrame: ...
    @overload
    def rename(self, mapper: Optional[Renamer] = ..., *, index: Optional[Renamer] = ..., columns: Optional[Renamer] = ..., axis: Optional[Axis] = ..., copy: bool = ..., inplace: Literal[True], level: Optional[Level] = ..., errors: ErrorsStrategy = ...) -> None: ...
    @overload
    def fillna(self, value: FillValue = ..., method: Optional[FillMethod] = ..., axis: Optional[Axis] = ...,  limit: int = ..., downcast: Optional[Dict[Any, Dtype]] = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def fillna(self, value: FillValue = ..., method: Optional[FillMethod] = ..., axis: Optional[Axis] = ..., limit: int = ..., downcast: Optional[Dict[Any, Dtype]] = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool = ..., method: ReplaceMethod = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool = ..., method: ReplaceMethod = ..., *,  inplace: Literal[True]) -> None: ...
    def shift(self, periods: int = ..., freq: Optional[Frequency] = ..., axis: AxisOption = ..., fill_value: Scalar = ...) -> DataFrame: ...
    @overload
    def set_index(self, keys: Union[Label, IndexArray, List[Union[Label, IndexArray]]], drop: bool = ..., append: bool = ..., verify_integrity: bool = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def set_index(self, keys: Union[Label, IndexArray, List[Union[Label, IndexArray]]], drop: bool = ..., append: bool = ..., verify_integrity: bool = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def reset_index(self, level: Optional[Union[Hashable, Sequence[Hashable]]] = ..., drop: bool = ..., col_level: Hashable = ..., col_fill: Optional[Hashable] = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def reset_index(self, level: Optional[Union[Hashable, Sequence[Hashable]]] = ..., drop: bool = ..., col_level: Hashable = ..., col_fill: Optional[Hashable] = ..., *, inplace: Literal[True]) -> None: ...
    def isna(self) -> DataFrame: ...
    def isnull(self) -> DataFrame: ...
    def notna(self) -> DataFrame: ...
    def notnull(self) -> DataFrame: ...
    @overload
    def dropna(self, axis: AxisOption = ..., how: DropTypes = ..., thresh: Optional[int] = ..., subset: Optional[Any] = ..., inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def dropna(self, axis: AxisOption = ..., how: DropTypes = ..., thresh: Optional[int] = ..., subset: Optional[Any] = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def drop_duplicates(self, subset: Optional[Union[Hashable, Sequence[Hashable]]] = ..., keep: GeneralDuplicatesKeepStrategy = ..., inplace: Literal[False] = ..., ignore_index: bool = ...) -> DataFrame: ...
    @overload
    def drop_duplicates(self, subset: Optional[Union[Hashable, Sequence[Hashable]]] = ..., keep: GeneralDuplicatesKeepStrategy = ..., ignore_index: bool = ..., *, inplace: Literal[True]) -> None: ...
    def duplicated(self, subset: Optional[Union[Hashable, Sequence[Hashable]]] = ..., keep: Union[str, bool] = ...) -> Series: ...
    @overload    # type: ignore[override]
    def sort_values(self, by: Union[str, List[str]], axis: AxisOption = ..., ascending: Union[List[bool], bool] = ..., kind: SortKind = ..., na_position: NaSortPosition = ..., ignore_index: bool = ...,  key: ValueKeyFunc = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def sort_values(self, by: Union[str, List[str]], axis: AxisOption = ..., ascending: Union[List[bool], bool] = ..., kind: SortKind = ..., na_position: NaSortPosition = ..., ignore_index: bool = ...,  key: ValueKeyFunc = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def sort_index(self, axis: AxisOption = ..., level: Optional[Union[Level, List[Level]]] = ..., ascending: bool = ..., kind: SortKind = ..., na_position: NaSortPosition = ..., sort_remaining: bool = ..., ignore_index: bool = ...,  key: IndexKeyFunc = ..., *, inplace: Literal[False] = ...) -> DataFrame: ...
    @overload
    def sort_index(self, axis: AxisOption = ..., level: Optional[Union[Level, List[Level]]] = ..., ascending: bool = ..., kind: SortKind = ..., na_position: NaSortPosition = ..., sort_remaining: bool = ..., ignore_index: bool = ...,  key: IndexKeyFunc = ..., *, inplace: Literal[True]) -> None: ...
    def nlargest(self, n: int, columns: Union[Label, List[Label]], keep: KeepStrategy = ...) -> DataFrame: ...
    def nsmallest(self, n: int, columns: Union[Label, List[Label]], keep: KeepStrategy = ...) -> DataFrame: ...
    def swaplevel(self, i: Level = ..., j: Level = ..., axis: AxisOption = ...) -> DataFrame: ...
    def reorder_levels(self, order: Union[List[int], List[str]], axis: AxisOption = ...) -> DataFrame: ...
    def combine(self, other: DataFrame, func: Union[np.func, Callable[[Series, Series], Union[Series, Scalar]]], fill_value: Optional[Scalar] = ..., overwrite: bool = ...) -> DataFrame: ...
    def combine_first(self, other: DataFrame) -> DataFrame: ...
    def update(self, other: Union[DataFrame, CoercibleIntoDataFrame], join: UpdateJoinType = ..., overwrite: bool = ..., filter_func: Optional[Callable[..., bool]] = ..., errors: UpdateErrorsStrategy = ...) -> None: ...
    def groupby(self, by: Optional[GroupByObject] = ..., axis: AxisOption = ..., level: Optional[Sequence[Level]] = ..., as_index: bool = ..., sort: bool = ..., group_keys: bool = ..., squeeze: bool = ..., observed: bool = ..., dropna: bool = ...) -> groupby_generic.DataFrameGroupBy: ...
    def pivot(self, index: Optional[Union[Label, Sequence[Label]]] = ..., columns: Optional[Union[Label, Sequence[Label]]] = ..., values: Optional[Union[Label, Sequence[Label]]] = ...) -> DataFrame: ...
    def pivot_table(self, values: Optional[Sequence[Column]] = ..., index: Optional[Union[Column, Grouper, np.ndarray, List[Union[Column, Grouper, np.ndarray]]]] = ..., columns: Optional[Union[Column, Grouper, np.ndarray, List[Union[Column, Grouper, np.ndarray]]]] = ..., aggfunc: AggregationFunction = ..., fill_value: Scalar = ..., margins: bool = ..., dropna: bool = ..., margins_name: str = ..., observed: bool = ...) -> DataFrame: ...
    def stack(self, level: Union[Level, List[Level]] = ..., dropna: bool = ...) -> FrameOrSeries: ...
    def explode(self, column: Union[Column, Tuple[Column, ...]]) -> DataFrame: ...
    def unstack(self, level: Union[Level, List[Level]] = ..., fill_value: Optional[Scalar] = ...) -> FrameOrSeries: ...
    def melt(self, id_vars: Optional[Union[Tuple[Column], List[Column], np.ndarray]] = ..., value_vars: Optional[Union[Sequence[Column], np.ndarray]] = ..., var_name: Optional[Scalar] = ..., value_name: Scalar = ..., col_level: Optional[Level] = ..., ignore_index: bool = ...) -> DataFrame: ...
    def diff(self, periods: int = ..., axis: AxisOption = ...) -> DataFrame: ...
    def aggregate(self, func: AggregationFunction = ..., axis: AxisOption = ..., *args: Any, **kwargs: Any) -> Union[Scalar, FrameOrSeries]: ...
    def agg(self, func: AggregationFunction = ..., axis: AxisOption = ..., *args: Any, **kwargs: Any) -> Union[Scalar, FrameOrSeries]: ...
    def transform(self, func: TransformFunction, axis: AxisOption = ..., *args: Any, **kwargs: Any) -> DataFrame: ...
    def apply(self, func: Function, axis: AxisOption = ..., raw: bool = ..., result_type: Optional[ApplyResultType] = ..., args: Any = ..., **kwds: Any) -> FrameOrSeries: ...
    def applymap(self, func: Callable[[Any], Any], na_action: Optional[str] = ...) -> DataFrame: ...
    def append(self, other: Union[FrameOrSeries, Dict[Column, Any], List[Union[FrameOrSeries, Dict[Column, Any]]]], ignore_index: bool = ..., verify_integrity: bool = ..., sort: bool = ...) -> DataFrame: ...
    def join(self, other: Union[FrameOrSeries, List[DataFrame]], on: Optional[Union[str, List[str], ArrayLike]] = ..., how: JoinType = ..., lsuffix: str = ..., rsuffix: str = ..., sort: bool = ...) -> DataFrame: ...
    def merge(self, right: FrameOrSeries, how: MergeType = ..., on: Optional[Union[Label, List[Label]]] = ..., left_on: Optional[Union[Label, List[Label], ArrayLike]] = ..., right_on: Optional[Union[Label, List[Label], ArrayLike]] = ..., left_index: bool = ..., right_index: bool = ..., sort: bool = ..., suffixes: Tuple[str, str] = ..., copy: bool = ..., indicator: Union[bool, str] = ..., validate: Optional[MergeValidationMethod] = ...) -> DataFrame: ...
    def round(self, decimals: Union[int, Dict[Column, int], Series] = ..., *args: Any, **kwargs: Any) -> DataFrame: ...
    def corr(self, method: CorrelationMethod = ..., min_periods: Optional[int] = ...) -> DataFrame: ...
    def cov(self, min_periods: Optional[int] = ..., ddof: Optional[int] = ...) -> DataFrame: ...
    def corrwith(self, other: FrameOrSeries, axis: AxisOption = ..., drop: bool = ..., method: CorrelationMethod = ...) -> Series: ...
    def count(self, axis: AxisOption = ..., level: Optional[Level] = ..., numeric_only: bool = ...) -> FrameOrSeries: ...
    def nunique(self, axis: AxisOption = ..., dropna: Optional[bool] = ...) -> Series: ...
    def idxmin(self, axis: AxisOption = ..., skipna: Optional[bool] = ...) -> Series: ...
    def idxmax(self, axis: AxisOption = ..., skipna: Optional[bool] = ...) -> Series: ...
    def mode(self, axis: AxisOption = ..., numeric_only: bool = ..., dropna: Optional[bool] = ...) -> DataFrame: ...
    def quantile(self, q: Union[float, ArrayLike] = ..., axis: AxisOption = ..., numeric_only: bool = ..., interpolation: InterpolationMethod = ...) -> FrameOrSeries: ...
    def to_timestamp(self, freq: Optional[str] = ..., how: TimestampMethod = ..., axis: AxisOption = ..., copy: bool = ...) -> DataFrame: ...
    def to_period(self, freq: Optional[str] = ..., axis: AxisOption = ..., copy: bool = ...) -> DataFrame: ...
    def isin(self, values: Union[Sequence[Scalar], FrameOrSeries, Dict[Column, Scalar], np.ndarray]) -> DataFrame: ...
    def compare(self, other: DataFrame, align_axis: AxisOption = ..., keep_shape: bool = ..., keep_equal: bool = ...) -> DataFrame: ...




from pandas import DataFrame as DataFrame

from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype
from typing import Any, Optional

loads: Any

def as_json_table_type(x: Any) -> Any: ...
def set_default_names(data: Any) -> Any: ...
def convert_pandas_type_to_json_field(arr: Any, dtype: Optional[Any] = ...) -> Any: ...
def convert_json_field_to_pandas_type(field: Any) -> Any: ...
def build_table_schema(data: Any, index: bool = ..., primary_key: Optional[Any] = ..., version: bool = ...) -> Any: ...
def parse_table_schema(json: Any, precise_float: Any) -> Any: ...



import numpy as np
from pandas._libs import missing as libmissing
from typing import Any, Optional, Union

def kleene_or(left: Union[bool, np.ndarray], right: Union[bool, np.ndarray], left_mask: Optional[np.ndarray], right_mask: Optional[np.ndarray]) -> Any: ...
def kleene_xor(left: Union[bool, np.ndarray], right: Union[bool, np.ndarray], left_mask: Optional[np.ndarray], right_mask: Optional[np.ndarray]) -> Any: ...
def kleene_and(left: Union[bool, libmissing.NAType, np.ndarray], right: Union[bool, libmissing.NAType, np.ndarray], left_mask: Optional[np.ndarray], right_mask: Optional[np.ndarray]) -> Any: ...
def raise_for_nan(value: Any, method: Any) -> None: ...



import numpy as np
from pandas._typing import FrameOrSeries as FrameOrSeries
from pandas.core.base import SelectionMixin as SelectionMixin
from pandas.core.dtypes.common import ensure_float64 as ensure_float64, ensure_int64 as ensure_int64, ensure_int_or_float as ensure_int_or_float, ensure_platform_int as ensure_platform_int, is_bool_dtype as is_bool_dtype, is_categorical_dtype as is_categorical_dtype, is_complex_dtype as is_complex_dtype, is_datetime64_any_dtype as is_datetime64_any_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_extension_array_dtype as is_extension_array_dtype, is_integer_dtype as is_integer_dtype, is_numeric_dtype as is_numeric_dtype, is_period_dtype as is_period_dtype, is_sparse as is_sparse, is_timedelta64_dtype as is_timedelta64_dtype, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.missing import isna as isna
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.generic import NDFrame as NDFrame
from pandas.core.groupby import base as base, grouper as grouper
from pandas.core.indexes.api import Index as Index, MultiIndex as MultiIndex, ensure_index as ensure_index
from pandas.core.series import Series as Series
from pandas.core.sorting import compress_group_index as compress_group_index, decons_obs_group_ids as decons_obs_group_ids, get_flattened_iterator as get_flattened_iterator, get_group_index as get_group_index, get_group_index_sorter as get_group_index_sorter, get_indexer_dict as get_indexer_dict
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, List, Optional, Sequence, Tuple

class BaseGrouper:
    axis: Any = ...
    sort: Any = ...
    group_keys: Any = ...
    mutated: Any = ...
    indexer: Any = ...
    def __init__(self, axis: Index, groupings: Sequence[grouper.Grouping], sort: bool=..., group_keys: bool=..., mutated: bool=..., indexer: Optional[np.ndarray]=...) -> None: ...
    @property
    def groupings(self) -> List[grouper.Grouping]: ...
    @property
    def shape(self) -> Any: ...
    def __iter__(self) -> Any: ...
    @property
    def nkeys(self) -> int: ...
    def get_iterator(self, data: FrameOrSeries, axis: int=...) -> Any: ...
    def apply(self, f: Any, data: FrameOrSeries, axis: int=...) -> Any: ...
    def indices(self) -> Any: ...
    @property
    def codes(self) -> List[np.ndarray]: ...
    @property
    def levels(self) -> List[Index]: ...
    @property
    def names(self) -> Any: ...
    def size(self) -> Series: ...
    def groups(self) -> Any: ...
    def is_monotonic(self) -> bool: ...
    def group_info(self) -> Any: ...
    def codes_info(self) -> np.ndarray: ...
    def ngroups(self) -> int: ...
    @property
    def reconstructed_codes(self) -> List[np.ndarray]: ...
    def result_index(self) -> Index: ...
    def get_group_levels(self) -> Any: ...
    def aggregate(self, values: Any, how: str, axis: int=..., min_count: int=...) -> Tuple[np.ndarray, Optional[List[str]]]: ...
    def transform(self, values: Any, how: str, axis: int=..., **kwargs: Any) -> Any: ...
    def agg_series(self, obj: Series, func: Any) -> Any: ...

class BinGrouper(BaseGrouper):
    bins: Any = ...
    binlabels: Any = ...
    mutated: Any = ...
    indexer: Any = ...
    def __init__(self, bins: Any, binlabels: Any, filter_empty: bool=..., mutated: bool=..., indexer: Any = ...) -> None: ...
    def groups(self) -> Any: ...
    @property
    def nkeys(self) -> int: ...
    def get_iterator(self, data: FrameOrSeries, axis: int=...) -> Any: ...
    def indices(self) -> Any: ...
    def group_info(self) -> Any: ...
    def reconstructed_codes(self) -> List[np.ndarray]: ...
    def result_index(self) -> Any: ...
    @property
    def levels(self) -> Any: ...
    @property
    def names(self) -> Any: ...
    @property
    def groupings(self) -> List[grouper.Grouping]: ...
    def agg_series(self, obj: Series, func: Any) -> Any: ...

class DataSplitter:
    data: Any = ...
    labels: Any = ...
    ngroups: Any = ...
    axis: Any = ...
    def __init__(self, data: FrameOrSeries, labels: Any, ngroups: int, axis: int=...) -> None: ...
    def slabels(self) -> Any: ...
    def sort_idx(self) -> Any: ...
    def __iter__(self) -> Any: ...

class SeriesSplitter(DataSplitter): ...

class FrameSplitter(DataSplitter):
    def fast_apply(self, f: Any, names: Any) -> Any: ...

def get_splitter(data: FrameOrSeries, *args: Any, **kwargs: Any) -> DataSplitter: ...



from pandas import Categorical as Categorical, CategoricalIndex as CategoricalIndex, DataFrame as DataFrame, DatetimeIndex as DatetimeIndex, Index as Index, IntervalIndex as IntervalIndex, MultiIndex as MultiIndex, RangeIndex as RangeIndex, Series as Series, bdate_range as bdate_range
from pandas._config.localization import can_set_locale as can_set_locale, get_locales as get_locales, set_locale as set_locale
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer, FrameOrSeries as FrameOrSeries
from pandas.core.algorithms import take_1d as take_1d
from pandas.core.arrays import DatetimeArray as DatetimeArray, ExtensionArray as ExtensionArray, IntervalArray as IntervalArray, PeriodArray as PeriodArray, TimedeltaArray as TimedeltaArray, period_array as period_array
from pandas.core.dtypes.missing import array_equivalent as array_equivalent
from pandas.io.common import urlopen as urlopen
from pandas.io.formats.printing import pprint_thing as pprint_thing
from typing import Any, List, Optional, Union, AnyStr

lzma: Any
N: int
K: int

def set_testing_mode() -> None: ...
def reset_testing_mode() -> None: ...
def reset_display_options() -> None: ...
def round_trip_pickle(obj: Any, path: Optional[FilePathOrBuffer[AnyStr]]=...) -> FrameOrSeries: ...
def round_trip_pathlib(writer: Any, reader: Any, path: Optional[str]=...) -> Any: ...
def round_trip_localpath(writer: Any, reader: Any, path: Optional[str]=...) -> Any: ...
def decompress_file(path: Any, compression: Any) -> None: ...
def write_to_compressed(compression: Any, path: Any, data: Any, dest: str = ...) -> None: ...
def assert_almost_equal(left: Any, right: Any, check_dtype: Union[bool, str]=..., check_less_precise: Union[bool, int]=..., **kwargs: Any) -> Any: ...
def assert_dict_equal(left: Any, right: Any, compare_keys: bool=...) -> Any: ...
def randbool(size: Any=..., p: float=...) -> Any: ...

RANDS_CHARS: Any
RANDU_CHARS: Any

def rands_array(nchars: Any, size: Any, dtype: str = ...) -> Any: ...
def randu_array(nchars: Any, size: Any, dtype: str = ...) -> Any: ...
def rands(nchars: Any) -> Any: ...
def randu(nchars: Any) -> Any: ...
def close(fignum: Optional[Any] = ...) -> None: ...
def ensure_clean(filename: Optional[Any] = ..., return_filelike: bool = ...) -> None: ...
def ensure_clean_dir() -> None: ...
def ensure_safe_environment_variables() -> None: ...
def equalContents(arr1: Any, arr2: Any) -> bool: ...
def assert_index_equal(left: Index, right: Index, exact: Union[bool, str]=..., check_names: bool=..., check_less_precise: Union[bool, int]=..., check_exact: bool=..., check_categorical: bool=..., obj: str=...) -> None: ...
def assert_class_equal(left: Any, right: Any, exact: Union[bool, str] = ..., obj: Any = ...) -> Any: ...
def assert_attr_equal(attr: Any, left: Any, right: Any, obj: str = ...) -> Any: ...
def assert_is_valid_plot_return_object(objs: Any) -> None: ...
def isiterable(obj: Any) -> Any: ...
def assert_is_sorted(seq: Any) -> None: ...
def assert_categorical_equal(left: Any, right: Any, check_dtype: bool = ..., check_category_order: bool = ..., obj: str = ...) -> None: ...
def assert_interval_array_equal(left: Any, right: Any, exact: str = ..., obj: str = ...) -> None: ...
def assert_period_array_equal(left: Any, right: Any, obj: str = ...) -> None: ...
def assert_datetime_array_equal(left: Any, right: Any, obj: str = ...) -> None: ...
def assert_timedelta_array_equal(left: Any, right: Any, obj: str = ...) -> None: ...
def raise_assert_detail(obj: Any, message: Any, left: Any, right: Any, diff: Optional[Any] = ...) -> None: ...
def assert_numpy_array_equal(left: Any, right: Any, strict_nan: bool = ..., check_dtype: bool = ..., err_msg: Optional[Any] = ..., check_same: Optional[Any] = ..., obj: str = ...) -> Any: ...
def assert_extension_array_equal(left: Any, right: Any, check_dtype: bool = ..., check_less_precise: bool = ..., check_exact: bool = ...) -> None: ...
def assert_series_equal(left: Any, right: Any, check_dtype: bool = ..., check_index_type: str = ..., check_series_type: bool = ..., check_less_precise: bool = ..., check_names: bool = ..., check_exact: bool = ..., check_datetimelike_compat: bool = ..., check_categorical: bool = ..., obj: str = ...) -> None: ...
def assert_frame_equal(left: Any, right: Any, check_dtype: bool = ..., check_index_type: str = ..., check_column_type: str = ..., check_frame_type: bool = ..., check_less_precise: bool = ..., check_names: bool = ..., by_blocks: bool = ..., check_exact: bool = ..., check_datetimelike_compat: bool = ..., check_categorical: bool = ..., check_like: bool = ..., obj: str = ...) -> None: ...
def assert_equal(left: Any, right: Any, **kwargs: Any) -> None: ...
def box_expected(expected: Any, box_cls: Any, transpose: bool = ...) -> Any: ...
def to_array(obj: Any) -> Any: ...
def assert_sp_array_equal(left: Any, right: Any, check_dtype: bool = ..., check_kind: bool = ..., check_fill_value: bool = ..., consolidate_block_indices: bool = ...) -> None: ...
def assert_contains_all(iterable: Any, dic: Any) -> None: ...
def assert_copy(iter1: Any, iter2: Any, **eql_kwargs: Any) -> None: ...
def getCols(k: Any) -> Any: ...
def makeStringIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeUnicodeIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeCategoricalIndex(k: int = ..., n: int = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makeIntervalIndex(k: int = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makeBoolIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeIntIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeUIntIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeRangeIndex(k: int = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makeFloatIndex(k: int = ..., name: Optional[Any] = ...) -> Any: ...
def makeDateIndex(k: int = ..., freq: str = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makeTimedeltaIndex(k: int = ..., freq: str = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makePeriodIndex(k: int = ..., name: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def makeMultiIndex(k: int = ..., names: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def all_index_generator(k: int = ...) -> None: ...
def index_subclass_makers_generator() -> None: ...
def all_timeseries_index_generator(k: int = ...) -> None: ...
def makeFloatSeries(name: Optional[Any] = ...) -> Any: ...
def makeStringSeries(name: Optional[Any] = ...) -> Any: ...
def makeObjectSeries(name: Optional[Any] = ...) -> Any: ...
def getSeriesData() -> Any: ...
def makeTimeSeries(nper: Optional[Any] = ..., freq: str = ..., name: Optional[Any] = ...) -> Any: ...
def makePeriodSeries(nper: Optional[Any] = ..., name: Optional[Any] = ...) -> Any: ...
def getTimeSeriesData(nper: Optional[Any] = ..., freq: str = ...) -> Any: ...
def getPeriodData(nper: Optional[Any] = ...) -> Any: ...
def makeTimeDataFrame(nper: Optional[Any] = ..., freq: str = ...) -> Any: ...
def makeDataFrame() -> Any: ...
def getMixedTypeDict() -> Any: ...
def makeMixedDataFrame() -> Any: ...
def makePeriodFrame(nper: Optional[Any] = ...) -> Any: ...
def makeCustomIndex(nentries: Any, nlevels: Any, prefix: str = ..., names: bool = ..., ndupe_l: Optional[Any] = ..., idx_type: Optional[Any] = ...) -> Any: ...
def makeCustomDataframe(nrows: Any, ncols: Any, c_idx_names: bool = ..., r_idx_names: bool = ..., c_idx_nlevels: int = ..., r_idx_nlevels: int = ..., data_gen_f: Optional[Any] = ..., c_ndupe_l: Optional[Any] = ..., r_ndupe_l: Optional[Any] = ..., dtype: Optional[Any] = ..., c_idx_type: Optional[Any] = ..., r_idx_type: Optional[Any] = ...) -> Any: ...
def makeMissingCustomDataframe(nrows: Any, ncols: Any, density: float = ..., random_state: Optional[Any] = ..., c_idx_names: bool = ..., r_idx_names: bool = ..., c_idx_nlevels: int = ..., r_idx_nlevels: int = ..., data_gen_f: Optional[Any] = ..., c_ndupe_l: Optional[Any] = ..., r_ndupe_l: Optional[Any] = ..., dtype: Optional[Any] = ..., c_idx_type: Optional[Any] = ..., r_idx_type: Optional[Any] = ...) -> Any: ...
def makeMissingDataframe(density: float = ..., random_state: Optional[Any] = ...) -> Any: ...
def optional_args(decorator: Any) -> Any: ...
def can_connect(url: Any, error_classes: Optional[Any] = ...) -> Any: ...
def network(t: Any, url: str = ..., raise_on_error: Any = ..., check_before_test: bool = ..., error_classes: Optional[Any] = ..., skip_errnos: Any = ..., _skip_on_messages: Any = ...) -> Any: ...
with_connectivity_check = network

def assert_produces_warning(expected_warning: Any = ..., filter_level: str = ..., clear: Optional[Any] = ..., check_stacklevel: bool = ..., raise_on_extra_warnings: bool = ...) -> None: ...

class RNGContext:
    seed: Any = ...
    def __init__(self, seed: Any) -> None: ...
    start_state: Any = ...
    def __enter__(self) -> None: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None: ...

def with_csv_dialect(name: Any, **kwargs: Any) -> None: ...
def use_numexpr(use: Any, min_elements: Optional[Any] = ...) -> None: ...
def test_parallel(num_threads: int = ..., kwargs_list: Optional[Any] = ...) -> Any: ...

class SubclassedSeries(Series): ...
class SubclassedDataFrame(DataFrame): ...
class SubclassedCategorical(Categorical): ...

def set_timezone(tz: str) -> Any: ...
def convert_rows_list_to_csv_str(rows_list: List[str]) -> Any: ...



import numpy as np
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional, Tuple

get_window_bounds_doc: str

class BaseIndexer:
    index_array: Any = ...
    window_size: Any = ...
    def __init__(self, index_array: Optional[np.ndarray]=..., window_size: int=..., **kwargs: Any) -> None: ...
    def get_window_bounds(self, num_values: int=..., min_periods: Optional[int]=..., center: Optional[bool]=..., closed: Optional[str]=...) -> Tuple[np.ndarray, np.ndarray]: ...

class FixedWindowIndexer(BaseIndexer):
    def get_window_bounds(self, num_values: int=..., min_periods: Optional[int]=..., center: Optional[bool]=..., closed: Optional[str]=...) -> Tuple[np.ndarray, np.ndarray]: ...

class VariableWindowIndexer(BaseIndexer):
    def get_window_bounds(self, num_values: int=..., min_periods: Optional[int]=..., center: Optional[bool]=..., closed: Optional[str]=...) -> Tuple[np.ndarray, np.ndarray]: ...

class ExpandingIndexer(BaseIndexer):
    def get_window_bounds(self, num_values: int=..., min_periods: Optional[int]=..., center: Optional[bool]=..., closed: Optional[str]=...) -> Tuple[np.ndarray, np.ndarray]: ...



import numpy as np
from pandas._config import get_option as get_option
from pandas._libs.interval import Interval as Interval, IntervalMixin as IntervalMixin, intervals_to_interval_bounds as intervals_to_interval_bounds
from pandas.core.algorithms import take as take, value_counts as value_counts
from pandas.core.arrays.base import ExtensionArray as ExtensionArray
from pandas.core.arrays.categorical import Categorical as Categorical
from pandas.core.construction import array as array
from pandas.core.dtypes.cast import maybe_convert_platform as maybe_convert_platform

from pandas.core.dtypes.dtypes import IntervalDtype as IntervalDtype
from pandas.core.dtypes.generic import ABCDatetimeIndex as ABCDatetimeIndex, ABCExtensionArray as ABCExtensionArray, ABCIndexClass as ABCIndexClass, ABCInterval as ABCInterval, ABCIntervalIndex as ABCIntervalIndex, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna, notna as notna

from pandas.core.indexes.base import ensure_index as ensure_index
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional

class IntervalArray(IntervalMixin, ExtensionArray):
    ndim: int = ...
    can_hold_na: bool = ...
    def __new__(cls, data: Any, closed: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ..., verify_integrity: bool = ...) -> Any: ...
    @classmethod
    def from_breaks(cls, breaks: Any, closed: str = ..., copy: bool = ..., dtype: Optional[Any] = ...) -> Any: ...
    @classmethod
    def from_arrays(cls, left: Any, right: Any, closed: str = ..., copy: bool = ..., dtype: Optional[Any] = ...) -> Any: ...
    @classmethod
    def from_tuples(cls, data: Any, closed: str = ..., copy: bool = ..., dtype: Optional[Any] = ...) -> Any: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __getitem__(self, value: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    @property
    def dtype(self) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def copy(self) -> Any: ...
    def isna(self) -> Any: ...
    @property
    def nbytes(self) -> int: ...
    @property
    def size(self) -> int: ...
    def shift(self, periods: int=..., fill_value: object=...) -> ABCExtensionArray: ...
    def take(self, indices: Any, allow_fill: bool = ..., fill_value: Optional[Any] = ..., axis: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...
    @property
    def left(self) -> Any: ...
    @property
    def right(self) -> Any: ...
    @property
    def closed(self) -> Any: ...
    def set_closed(self, closed: Any) -> Any: ...
    @property
    def length(self) -> Any: ...
    @property
    def mid(self) -> Any: ...
    @property
    def is_non_overlapping_monotonic(self) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __arrow_array__(self, type: Optional[Any] = ...) -> Any: ...
    def to_tuples(self, na_tuple: bool = ...) -> Any: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def contains(self, other: Any) -> Any: ...
    def overlaps(self, other: Any) -> Any: ...

def maybe_convert_platform_interval(values: Any) -> Any: ...



from pandas import get_option as get_option, option_context as option_context
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame
from typing import Any, Optional

def read_clipboard(sep: str = ..., **kwargs: Any) -> Any: ...
def to_clipboard(obj: Any, excel: bool = ..., sep: Optional[Any] = ..., **kwargs: Any) -> None: ...



import numpy as np
from datetime import tzinfo
from pandas._libs.tslibs import parsing as parsing, timezones as timezones
from pandas.core.accessor import delegate_names as delegate_names
from pandas.core.arrays.datetimes import DatetimeArray as DatetimeArray, tz_to_dtype as tz_to_dtype, validate_tz_from_dtype as validate_tz_from_dtype
from pandas.core.dtypes.dtypes import DatetimeTZDtype as DatetimeTZDtype
from pandas.core.dtypes.missing import is_valid_nat_for_dtype as is_valid_nat_for_dtype, isna as isna
from pandas.core.indexes.base import Index as Index, maybe_extract_name as maybe_extract_name
from pandas.core.indexes.datetimelike import DatetimeTimedeltaMixin as DatetimeTimedeltaMixin, DatetimelikeDelegateMixin as DatetimelikeDelegateMixin
from pandas.core.indexes.extension import inherit_names as inherit_names
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.tseries.frequencies import Resolution as Resolution, to_offset as to_offset
from pandas.tseries.offsets import Nano as Nano, prefix_mapping as prefix_mapping
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from typing import Any, Optional

class DatetimeDelegateMixin(DatetimelikeDelegateMixin): ...

class DatetimeIndex(DatetimeTimedeltaMixin, DatetimeDelegateMixin):
    tz: Optional[tzinfo]
    def __new__(cls, data: Optional[Any] = ..., freq: Optional[Any] = ..., tz: Optional[Any] = ..., normalize: bool = ..., closed: Optional[Any] = ..., ambiguous: str = ..., dayfirst: bool = ..., yearfirst: bool = ..., dtype: Optional[Any] = ..., copy: bool = ..., name: Optional[Any] = ...) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __reduce__(self) -> Any: ...
    def union_many(self, others: Any) -> Any: ...
    def to_series(self, keep_tz: Any = ..., index: Optional[Any] = ..., name: Optional[Any] = ...) -> Any: ...  # type: ignore
    def snap(self, freq: str = ...) -> Any: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def get_value_maybe_box(self, series: Any, key: Any) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def slice_indexer(self, start: Optional[Any] = ..., end: Optional[Any] = ..., step: Optional[Any] = ..., kind: Optional[Any] = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def is_type_compatible(self, typ: Any) -> bool: ...
    @property
    def inferred_type(self) -> str: ...
    def insert(self, loc: Any, item: Any) -> Any: ...
    def indexer_at_time(self, time: Any, asof: bool = ...) -> Any: ...
    def indexer_between_time(self, start_time: Any, end_time: Any, include_start: bool = ..., include_end: bool = ...) -> Any: ...

def date_range(start: Any = ..., end: Any = ..., periods: Any = ..., freq: Any = ..., tz: Any = ..., normalize: Any = ..., name: Any = ..., closed: Any = ..., **kwargs: Any) -> DatetimeIndex: ...
def bdate_range(start: Any = ..., end: Any = ..., periods: Any = ..., freq: Any = ..., tz: Any = ..., normalize: Any = ..., name: Any = ..., weekmask: Any = ..., holidays: Any = ..., closed: Any = ..., **kwargs: Any) -> DatetimeIndex: ...



import matplotlib.units as dates
import matplotlib.units as units
from matplotlib.ticker import Formatter, Locator

from typing import Any, Optional

HOURS_PER_DAY: float
MIN_PER_HOUR: float
SEC_PER_MIN: float
SEC_PER_HOUR: Any
SEC_PER_DAY: Any
MUSEC_PER_DAY: Any


def get_pairs() -> Any: ...
def register_pandas_matplotlib_converters(func: Any) -> Any: ...
def pandas_converters() -> None: ...
def register() -> None: ...
def deregister() -> None: ...
def time2num(d: Any) -> Any: ...

class TimeConverter(units.ConversionInterface): # type: ignore
    @staticmethod
    def convert(value: Any, unit: Any, axis: Any) -> Any: ...
    @staticmethod
    def axisinfo(unit: Any, axis: Any) -> Any: ...
    @staticmethod
    def default_units(x: Any, axis: Any) -> Any: ...

class TimeFormatter(Formatter): # type: ignore
    locs: Any = ...
    def __init__(self, locs: Any) -> None: ...
    def __call__(self, x: Any, pos: int = ...) -> Any: ...

class PeriodConverter(dates.DateConverter): # type: ignore
    @staticmethod
    def convert(values: Any, units: Any, axis: Any) -> Any: ...

def get_datevalue(date: Any, freq: Any) -> Any: ...

class DatetimeConverter(dates.DateConverter): # type: ignore
    @staticmethod
    def convert(values: Any, unit: Any, axis: Any) -> Any: ...
    @staticmethod
    def axisinfo(unit: Any, axis: Any) -> Any: ...

class PandasAutoDateFormatter(dates.AutoDateFormatter): # type: ignore
    def __init__(self, locator: Any, tz: Optional[Any] = ..., defaultfmt: str = ...) -> None: ...

class PandasAutoDateLocator(dates.AutoDateLocator): # type: ignore
    def get_locator(self, dmin: Any, dmax: Any) -> Any: ...

class MilliSecondLocator(dates.DateLocator): # type: ignore
    UNIT: Any = ...
    def __init__(self, tz: Any) -> None: ...
    @staticmethod
    def get_unit_generic(freq: Any) -> Any: ...
    def __call__(self) -> Any: ...
    def autoscale(self) -> Any: ...

def period_break(dates: Any, period: Any) -> Any: ...
def has_level_label(label_flags: Any, vmin: Any) -> Any: ...
def get_finder(freq: Any) -> Any: ...

class TimeSeries_DateLocator(Locator): # type: ignore
    freq: Any = ...
    base: Any = ...
    isminor: Any = ...
    isdynamic: Any = ...
    offset: int = ...
    plot_obj: Any = ...
    finder: Any = ...
    def __init__(self, freq: Any, minor_locator: bool = ..., dynamic_mode: bool = ..., base: int = ..., quarter: int = ..., month: int = ..., day: int = ..., plot_obj: Optional[Any] = ...) -> None: ...
    def __call__(self) -> Any: ...
    def autoscale(self) -> Any: ...

class TimeSeries_DateFormatter(Formatter): # type: ignore
    format: Any = ...
    freq: Any = ...
    locs: Any = ...
    formatdict: Any = ...
    isminor: Any = ...
    isdynamic: Any = ...
    offset: int = ...
    plot_obj: Any = ...
    finder: Any = ...
    def __init__(self, freq: Any, minor_locator: bool = ..., dynamic_mode: bool = ..., plot_obj: Optional[Any] = ...) -> None: ...
    def set_locs(self, locs: Any) -> None: ...
    def __call__(self, x: Any, pos: int = ...) -> Any: ...

class TimeSeries_TimedeltaFormatter(Formatter): # type: ignore
    @staticmethod
    def format_timedelta_ticks(x: Any, pos: Any, n_decimals: Any) -> Any: ...
    def __call__(self, x: Any, pos: int = ...) -> Any: ...



from typing import Any, Iterable, Optional

class _DeprecatedModule:
    deprmod: Any = ...
    deprmodto: Any = ...
    removals: Any = ...
    moved: Any = ...
    self_dir: Any = ...
    def __init__(self, deprmod: Any, deprmodto: Optional[Any] = ..., removals: Optional[Any] = ..., moved: Optional[Any] = ...) -> None: ...
    def __dir__(self) -> Iterable[str]: ...
    def __getattr__(self, name: str) -> Any: ...



from typing import Any

import numpy

class Period:
    ...


def get_period_field_arr(code: int, arr: Any, freq: int) -> Any: ...


def period_asfreq_arr(arr: numpy.ndarray, freq1: int, freq2: int, end: Any) -> Any: ...



from __future__ import annotations

import sys
from typing import Any, Callable, Dict, Hashable, List, Mapping, Optional, Sequence, Tuple, TypeVar, Union, AnyStr, overload
from pandas.core.resample import Resampler

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

import numpy as np
from pandas._libs.tslibs.timedeltas import Timedelta
from pandas._libs.tslibs.timestamps import Timestamp

import pandas.core.indexing as indexing
from pandas._typing import Axis, Dtype, FilePathOrBuffer, FrameOrSeries, JSONSerializable, Level, Renamer, \
    ReplaceMethod, ReplaceValue, ToReplace, Frequency, Scalar, AxisOption, SortKind, JoinType, NaSortPosition, \
    ErrorsStrategy, ArrayLike, ValueKeyFunc, IndexKeyFunc
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.core.base import PandasObject, SelectionMixin
from pandas.core.indexes.api import Index
from pandas.core.internals import BlockManager

bool_t = bool

Self = TypeVar('Self', bound=NDFrame)
PipeReturn = TypeVar('PipeReturn')

class NDFrame(PandasObject, SelectionMixin, indexing.IndexingMixin):
    __array_priority__: int = ...
    __bool__: Any = ...

    def __init__(self, data: BlockManager, axes: Optional[List[Index]]=..., copy: bool_t=..., dtype: Optional[Dtype]=..., attrs: Optional[Mapping[Optional[Hashable], Any]]=..., fastpath: bool_t=...) -> None: ...
    def __neg__(self) -> Any: ...
    def __pos__(self) -> Any: ...
    def __invert__(self) -> Any: ...
    def __abs__(self) -> FrameOrSeries: ...
    def __round__(self, decimals: int=...) -> FrameOrSeries: ...
    def __hash__(self) -> Any: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: Any) -> bool_t: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __delitem__(self, key: Any) -> None: ...
    def __copy__(self, deep: bool_t=...) -> FrameOrSeries: ...
    def __deepcopy__(self, memo: Any = ...) -> FrameOrSeries: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __array_wrap__(self, result: Any, context: Optional[Any] = ...) -> Any: ...
    def keys(self) -> Any: ...
    def items(self) -> Any: ...
    def iteritems(self) -> Any: ...
    @property
    def attrs(self) -> Dict[Optional[Hashable], Any]: ...
    @attrs.setter
    def attrs(self, value: Mapping[Optional[Hashable], Any]) -> None: ...
    @property
    def shape(self) -> Tuple[int, ...]: ...
    @property
    def axes(self) -> List[Index]: ...
    @property
    def ndim(self) -> int: ...
    @property
    def size(self) -> Any: ...
    def __nonzero__(self) -> None: ...
    def bool(self) -> Any: ...
    def set_axis(self, labels: Any, axis: int = ..., inplace: bool_t = ...) -> Any: ...
    def swapaxes(self, axis1: Any, axis2: Any, copy: Any = ...) -> FrameOrSeries: ...
    def droplevel(self, level: Any, axis: Any = ...) -> FrameOrSeries: ...
    def pop(self, item: Any) -> FrameOrSeries: ...
    def squeeze(self, axis: Optional[Any] = ...) -> Any: ...
    def swaplevel(self, i: Any = ..., j: Any = ..., axis: Any = ...) -> FrameOrSeries: ...
    @overload
    def rename(self, mapper: Optional[Renamer]=..., *, index: Optional[Renamer]=..., columns: Optional[Renamer]=..., axis: Optional[Axis]=..., copy: bool_t=..., inplace: Literal[False] = ..., level: Optional[Level]=..., errors: ErrorsStrategy=...) -> FrameOrSeries: ...
    @overload
    def rename(self, mapper: Optional[Renamer]=..., *, index: Optional[Renamer]=..., columns: Optional[Renamer]=..., axis: Optional[Axis]=..., copy: bool_t=..., inplace: Literal[True], level: Optional[Level]=..., errors: ErrorsStrategy=...) -> None: ...
    def rename_axis(self, mapper: Any = ..., **kwargs: Any) -> Optional[FrameOrSeries]: ...
    def equals(self, other: Any) -> Any: ...
    @property
    def empty(self) -> bool_t: ...
    def to_excel(self, excel_writer: Any, sheet_name: Any = ..., na_rep: Any = ..., float_format: Any = ..., columns: Any = ..., header: Any = ..., index: Any = ..., index_label: Any = ..., startrow: Any = ..., startcol: Any = ..., engine: Any = ..., merge_cells: Any = ..., encoding: Any = ..., inf_rep: Any = ..., verbose: Any = ..., freeze_panes: Any = ..., storage_options: Optional[Dict[str, Any]] = None) -> None: ...
    def to_json(self, path_or_buf: Optional[FilePathOrBuffer[AnyStr]]=..., orient: Optional[str]=..., date_format: Optional[str]=..., double_precision: int=..., force_ascii: bool_t=..., date_unit: str=..., default_handler: Optional[Callable[[Any], JSONSerializable]]=..., lines: bool_t=..., compression: Optional[str]=..., index: bool_t=..., indent: Optional[int]=..., storage_options: Optional[Dict[str, Any]] = None) -> Optional[str]: ...
    def to_hdf(self, path_or_buf: Any, key: str, mode: str=..., complevel: Optional[int]=..., complib: Optional[str]=..., append: bool_t=..., format: Optional[str]=..., index: bool_t=..., min_itemsize: Optional[Union[int, Dict[str, int]]]=..., nan_rep: Any = ..., dropna: Optional[bool_t]=..., data_columns: Optional[List[str]]=..., errors: str=..., encoding: str=...) -> None: ...
    def to_sql(self, name: str, con: Any, schema: Any = ..., if_exists: str=..., index: bool_t=..., index_label: Any = ..., chunksize: Any = ..., dtype: Any = ..., method: Any = ...) -> None: ...
    def to_pickle(self, path: Any, compression: Optional[str]=..., protocol: int=..., storage_options: Optional[Dict[str, Any]] = None) -> None: ...
    def to_clipboard(self, excel: bool_t=..., sep: Optional[str]=..., **kwargs: Any) -> None: ...
    def to_xarray(self) -> Any: ...
    def to_latex(self, buf: Optional[Any] = ..., columns: Optional[Any] = ..., col_space: Optional[Any] = ..., header: bool_t = ..., index: bool_t = ..., na_rep: str = ..., formatters: Optional[Any] = ..., float_format: Optional[Any] = ..., sparsify: Optional[Any] = ..., index_names: bool_t = ..., bold_rows: bool_t = ..., column_format: Optional[Any] = ..., longtable: Optional[Any] = ..., escape: Optional[Any] = ..., encoding: Optional[Any] = ..., decimal: str = ..., multicolumn: Optional[Any] = ..., multicolumn_format: Optional[Any] = ..., multirow: Optional[Any] = ..., caption: Union[str, Tuple[str, str]] = ..., label: Optional[Any] = ..., position: Optional[str] = ...) -> Any: ...
    def to_csv(self, path_or_buf: Optional[FilePathOrBuffer[AnyStr]]=..., sep: str=..., na_rep: str=..., float_format: Optional[str]=..., columns: Optional[Sequence[Optional[Hashable]]]=..., header: Union[bool_t, List[str]]=..., index: bool_t=..., index_label: Optional[Union[bool_t, str, Sequence[Optional[Hashable]]]]=..., mode: str=..., encoding: Optional[str]=..., compression: Optional[Union[str, Mapping[str, str]]]=..., quoting: Optional[int]=..., quotechar: str=..., line_terminator: Optional[str]=..., chunksize: Optional[int]=..., date_format: Optional[str]=..., doublequote: bool_t=..., escapechar: Optional[str]=..., decimal: Optional[str]=..., errors: str=..., storage_options: Optional[Dict[str, Any]] = None) -> Optional[str]: ...
    def take(self, indices: Any, axis: Any = ..., is_copy: Optional[bool_t]=..., **kwargs: Any) -> FrameOrSeries: ...
    def xs(self, key: Any, axis: Any = ..., level: Any = ..., drop_level: bool_t=...) -> Any: ...
    def get(self, key: Any, default: Optional[Any] = ...) -> Any: ...
    def reindex_like(self, other: Any, method: Optional[str]=..., copy: bool_t=..., limit: Any = ..., tolerance: Any = ...) -> FrameOrSeries: ...
    @overload
    def drop(self, labels: Any = ..., axis: Any = ..., index: Any = ..., columns: Any = ..., level: Any = ..., errors: ErrorsStrategy=..., *, inplace: Literal[False] = ...) -> Any: ...
    @overload
    def drop(self, labels: Any = ..., axis: Any = ..., index: Any = ..., columns: Any = ..., level: Any = ..., errors: ErrorsStrategy=..., *, inplace: Literal[True]) -> None: ...
    def add_prefix(self, prefix: str) -> FrameOrSeries: ...
    def add_suffix(self, suffix: str) -> FrameOrSeries: ...
    @overload
    def sort_values(self, by: Union[str, List[str]] = ..., axis: Union[int, str] = ..., ascending: bool_t = ..., kind: str = ..., na_position: str = ..., ignore_index: bool_t = ...,  key: ValueKeyFunc = ..., *, inplace: Literal[False] = ...) -> FrameOrSeries: ...
    @overload
    def sort_values(self, by: Union[str, List[str]] = ..., axis: Union[int, str] = ..., ascending: bool_t = ..., kind: str = ..., na_position: str = ..., ignore_index: bool_t = ...,  key: ValueKeyFunc = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def sort_index(self, axis: Any = ..., level: Any = ..., ascending: bool_t=..., kind: SortKind = ..., na_position: NaSortPosition = ..., sort_remaining: bool_t=..., ignore_index: bool_t=..., key: IndexKeyFunc = ..., *, inplace: Literal[False] = ...) -> Any: ...
    @overload
    def sort_index(self, axis: Any = ..., level: Any = ..., ascending: bool_t=..., kind: SortKind = ..., na_position: NaSortPosition = ..., sort_remaining: bool_t=..., ignore_index: bool_t=..., key: IndexKeyFunc = ..., *, inplace: Literal[True]) -> None: ...
    def reindex(self, *args: Any, **kwargs: Any) -> FrameOrSeries: ...
    def filter(self, items: Any = ..., like: Optional[str]=..., regex: Optional[str]=..., axis: Any = ...) -> FrameOrSeries: ...
    def head(self, n: int = ...) -> FrameOrSeries: ...
    def tail(self, n: int = ...) -> FrameOrSeries: ...
    def sample(self, n: int = ..., frac: float = ..., replace: bool_t = ..., weights: Union[str, ArrayLike] = ..., random_state: Union[int, np.random.RandomState] = ..., axis: Optional[AxisOption] = ...) -> FrameOrSeries: ...
    @overload
    def pipe(self: Any, func: Union[Callable[..., PipeReturn], Tuple[Callable[..., PipeReturn], str]], *args: Any, **kwargs: Any) -> PipeReturn: ...
    @overload
    def pipe(self: Any, func: PipeReturn, *args: Any, **kwargs: Any) -> PipeReturn: ...
    def __finalize__(self, other: Any, method: Any = ..., **kwargs: Any) -> FrameOrSeries: ...
    @property
    def values(self) -> np.ndarray: ...
    @property
    def dtypes(self) -> Any: ...
    def astype(self, dtype: Any, copy: bool_t=..., errors: str=...) -> FrameOrSeries: ...
    def copy(self: Self, deep: bool_t=...) -> Self: ...
    def infer_objects(self) -> FrameOrSeries: ...
    def convert_dtypes(self, infer_objects: bool_t=..., convert_string: bool_t=..., convert_integer: bool_t=..., convert_boolean: bool_t=...) -> FrameOrSeries: ...
    @overload
    def fillna(self, value: Any = ..., method: Any = ..., axis: Any = ..., limit: Any = ..., downcast: Any = ..., *, inplace: Literal[False] = ...) -> FrameOrSeries: ...
    @overload
    def fillna(self, value: Any = ..., method: Any = ..., axis: Any = ..., limit: Any = ..., downcast: Any = ..., *, inplace: Literal[True]) -> None: ...
    def ffill(self, axis: Any = ..., inplace: bool_t=..., limit: Any = ..., downcast: Any = ...) -> Optional[FrameOrSeries]: ...
    def bfill(self, axis: Any = ..., inplace: bool_t=..., limit: Any = ..., downcast: Any = ...) -> Optional[FrameOrSeries]: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool_t = ..., method: ReplaceMethod = ..., *, inplace: Literal[False] = ...) -> FrameOrSeries: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool_t = ..., method: ReplaceMethod = ..., *, inplace: Literal[True]) -> None: ...
    def interpolate(self, method: str = ..., axis: int = ..., limit: Optional[Any] = ..., inplace: bool_t = ..., limit_direction: str = ..., limit_area: Optional[Any] = ..., downcast: Optional[Any] = ..., **kwargs: Any) -> Optional[FrameOrSeries]: ...
    def asof(self, where: Any, subset: Optional[Any] = ...) -> Any: ...
    def isna(self) -> FrameOrSeries: ...
    def isnull(self) -> FrameOrSeries: ...
    def notna(self) -> FrameOrSeries: ...
    def notnull(self) -> FrameOrSeries: ...
    def clip(self, lower: Any = ..., upper: Any = ..., axis: Any = ..., inplace: bool_t=..., *args: Any, **kwargs: Any) -> Optional[FrameOrSeries]: ...
    def asfreq(self, freq: Any, method: Any = ..., how: Optional[str]=..., normalize: bool_t=..., fill_value: Any = ...) -> FrameOrSeries: ...
    def at_time(self, time: Any, asof: bool_t=..., axis: Any = ...) -> FrameOrSeries: ...
    def between_time(self, start_time: Any, end_time: Any, include_start: bool_t=..., include_end: bool_t=..., axis: Any = ...) -> FrameOrSeries: ...
    def resample(self, rule: Any, axis: Any = ..., closed: Optional[str]=..., label: Optional[str]=..., convention: str=..., kind: Optional[str]=..., loffset: Any = ..., base: int=..., on: Any = ..., level: Any = ..., origin: Union[Timestamp, str] = ..., offset: Union[Timedelta, str] = ...) -> Resampler: ...
    def first(self, offset: Any) -> FrameOrSeries: ...
    def last(self, offset: Any) -> FrameOrSeries: ...
    def rank(self, axis: Any = ..., method: str=..., numeric_only: Optional[bool_t]=..., na_option: str=..., ascending: bool_t=..., pct: bool_t=...) -> FrameOrSeries: ...
    def align(self, other: Any, join: JoinType = ..., axis: Optional[Any] = ..., level: Optional[Any] = ..., copy: bool_t = ..., fill_value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[int] = ..., fill_axis: AxisOption = ..., broadcast_axis: Optional[AxisOption] = ...) -> Any: ...
    def where(self, cond: Any, other: Any = ..., inplace: bool_t = ..., axis: Optional[Any] = ..., level: Optional[Any] = ..., errors: str = ..., try_cast: bool_t = ...) -> Any: ...
    def mask(self, cond: Any, other: Any = ..., inplace: bool_t = ..., axis: Optional[Any] = ..., level: Optional[Any] = ..., errors: str = ..., try_cast: bool_t = ...) -> Any: ...
    def shift(self, periods: int = ..., freq: Optional[Frequency] = ..., axis: AxisOption = ..., fill_value: Scalar = ...) -> FrameOrSeries: ...
    def slice_shift(self, periods: int=..., axis: Any = ...) -> FrameOrSeries: ...
    def tshift(self, periods: int=..., freq: Any = ..., axis: Any = ...) -> FrameOrSeries: ...
    def truncate(self, before: Any = ..., after: Any = ..., axis: Any = ..., copy: bool_t=...) -> FrameOrSeries: ...
    def tz_convert(self, tz: Any, axis: Any = ..., level: Any = ..., copy: bool_t=...) -> FrameOrSeries: ...
    def tz_localize(self, tz: Any, axis: Any = ..., level: Any = ..., copy: bool_t=..., ambiguous: Any = ..., nonexistent: str=...) -> FrameOrSeries: ...
    def abs(self) -> FrameOrSeries: ...
    def describe(self, percentiles: Any = ..., include: Any = ..., exclude: Any = ..., datetime_is_numeric: bool_t = ...) -> FrameOrSeries: ...
    def pct_change(self, periods: Any = ..., fill_method: Any = ..., limit: Any = ..., freq: Any = ..., **kwargs: Any) -> FrameOrSeries: ...
    def transform(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def first_valid_index(self) -> Any: ...
    def last_valid_index(self) -> Any: ...
    def set_flags(self, *, copy: bool_t = ..., allows_duplicate_labels: Optional[bool_t] = ...) -> FrameOrSeries: ...



from pandas import DataFrame as DataFrame
from pandas.core.construction import extract_array as extract_array
from pandas.core.dtypes.common import is_list_like as is_list_like, is_timedelta64_dtype as is_timedelta64_dtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCExtensionArray as ABCExtensionArray, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.ops.array_ops import arithmetic_op as arithmetic_op, comp_method_OBJECT_ARRAY as comp_method_OBJECT_ARRAY, comparison_op as comparison_op, define_na_arithmetic_op as define_na_arithmetic_op, get_array_op as get_array_op, logical_op as logical_op
from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.core.ops.dispatch import should_series_dispatch as should_series_dispatch
from pandas.core.ops.invalid import invalid_comparison as invalid_comparison
from pandas.core.ops.mask_ops import kleene_and as kleene_and, kleene_or as kleene_or, kleene_xor as kleene_xor
from pandas.core.ops.methods import add_flex_arithmetic_methods as add_flex_arithmetic_methods, add_special_arithmetic_methods as add_special_arithmetic_methods
from pandas.core.ops.roperator import radd as radd, rand_ as rand_, rdiv as rdiv, rdivmod as rdivmod, rfloordiv as rfloordiv, rmod as rmod, rmul as rmul, ror_ as ror_, rpow as rpow, rsub as rsub, rtruediv as rtruediv, rxor as rxor
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional, Set, Tuple

ARITHMETIC_BINOPS: Set[str]
COMPARISON_BINOPS: Set[str]

def get_op_result_name(left: Any, right: Any) -> Any: ...
def maybe_upcast_for_op(obj: Any, shape: Tuple[int, ...]) -> Any: ...
def fill_binop(left: Any, right: Any, fill_value: Any) -> Any: ...
def dispatch_to_series(left: Any, right: Any, func: Any, str_rep: Optional[Any] = ..., axis: Optional[Any] = ...) -> Any: ...



from pandas._libs.tslibs.resolution import Resolution as Resolution
from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.tseries.offsets import DateOffset as DateOffset
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Optional

def get_period_alias(offset_str: str) -> Optional[str]: ...
def to_offset(freq: Any) -> Optional[DateOffset]: ...
def get_offset(name: str) -> DateOffset: ...
def infer_freq(index: Any, warn: bool=...) -> Optional[str]: ...

class _FrequencyInferer:
    index: Any = ...
    values: Any = ...
    warn: Any = ...
    is_monotonic: Any = ...
    def __init__(self, index: Any, warn: bool=...) -> None: ...
    def deltas(self) -> Any: ...
    def deltas_asi8(self) -> Any: ...
    def is_unique(self) -> bool: ...
    def is_unique_asi8(self) -> Any: ...
    def get_freq(self) -> Optional[str]: ...
    def day_deltas(self) -> Any: ...
    def hour_deltas(self) -> Any: ...
    def fields(self) -> Any: ...
    def rep_stamp(self) -> Any: ...
    def month_position_check(self) -> Any: ...
    def mdiffs(self) -> Any: ...
    def ydiffs(self) -> Any: ...

class _TimedeltaFrequencyInferer(_FrequencyInferer): ...



import abc
from pandas._config import config as config

from pandas.core.frame import DataFrame as DataFrame
from pandas.errors import EmptyDataError as EmptyDataError
from pandas.io.excel._util import get_writer as get_writer
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.io.parsers import TextParser as TextParser
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional, Sequence, Union


def read_excel(io: Any, sheet_name: Optional[Union[str, int, Sequence[Union[str, int]]]] = ..., header: Optional[Union[int, Sequence[int]]] = ..., names: Optional[Any] = ..., index_col: Optional[Any] = ..., usecols: Optional[Any] = ..., squeeze: bool = ..., dtype: Optional[Any] = ..., engine: Optional[Any] = ..., converters: Optional[Any] = ..., true_values: Optional[Any] = ..., false_values: Optional[Any] = ..., skiprows: Optional[Any] = ..., nrows: Optional[Any] = ..., na_values: Optional[Any] = ..., keep_default_na: bool = ..., verbose: bool = ..., parse_dates: bool = ..., date_parser: Optional[Any] = ..., thousands: Optional[Any] = ..., comment: Optional[Any] = ..., skipfooter: int = ..., convert_float: bool = ..., mangle_dupe_cols: bool = ..., **kwds: Any) -> Any: ...

class _BaseExcelReader(metaclass=abc.ABCMeta):
    book: Any = ...
    def __init__(self, filepath_or_buffer: Any) -> None: ...
    @abc.abstractmethod
    def load_workbook(self, filepath_or_buffer: Any) -> Any: ...
    def close(self) -> None: ...
    @property
    @abc.abstractmethod
    def sheet_names(self) -> Any: ...
    @abc.abstractmethod
    def get_sheet_by_name(self, name: Any) -> Any: ...
    @abc.abstractmethod
    def get_sheet_by_index(self, index: Any) -> Any: ...
    @abc.abstractmethod
    def get_sheet_data(self, sheet: Any, convert_float: Any) -> Any: ...
    def parse(self, sheet_name: int = ..., header: int = ..., names: Optional[Any] = ..., index_col: Optional[Any] = ..., usecols: Optional[Any] = ..., squeeze: bool = ..., dtype: Optional[Any] = ..., true_values: Optional[Any] = ..., false_values: Optional[Any] = ..., skiprows: Optional[Any] = ..., nrows: Optional[Any] = ..., na_values: Optional[Any] = ..., verbose: bool = ..., parse_dates: bool = ..., date_parser: Optional[Any] = ..., thousands: Optional[Any] = ..., comment: Optional[Any] = ..., skipfooter: int = ..., convert_float: bool = ..., mangle_dupe_cols: bool = ..., **kwds: Any) -> Any: ...

class ExcelWriter(metaclass=abc.ABCMeta):
    def __new__(cls, path: Any, engine: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    book: Any = ...
    curr_sheet: Any = ...
    path: Any = ...
    @property
    @abc.abstractmethod
    def supported_extensions(self) -> Any: ...
    @property
    @abc.abstractmethod
    def engine(self) -> Any: ...
    @abc.abstractmethod
    def write_cells(self, cells: Any, sheet_name: Optional[Any] = ..., startrow: int = ..., startcol: int = ..., freeze_panes: Optional[Any] = ...) -> Any: ...
    @abc.abstractmethod
    def save(self) -> Any: ...
    sheets: Any = ...
    cur_sheet: Any = ...
    date_format: str = ...
    datetime_format: str = ...
    mode: Any = ...
    def __init__(self, path: Any, engine: Optional[Any] = ..., date_format: Optional[Any] = ..., datetime_format: Optional[Any] = ..., mode: str = ..., **engine_kwargs: Any) -> None: ...
    def __fspath__(self) -> Any: ...
    @classmethod
    def check_extension(cls, ext: Any) -> Any: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None: ...
    def close(self) -> Any: ...

class ExcelFile:
    engine: Any = ...
    io: Any = ...
    def __init__(self, io: Any, engine: Optional[Any] = ...) -> None: ...
    def __fspath__(self) -> Any: ...
    def parse(self, sheet_name: int = ..., header: int = ..., names: Optional[Any] = ..., index_col: Optional[Any] = ..., usecols: Optional[Any] = ..., squeeze: bool = ..., converters: Optional[Any] = ..., true_values: Optional[Any] = ..., false_values: Optional[Any] = ..., skiprows: Optional[Any] = ..., nrows: Optional[Any] = ..., na_values: Optional[Any] = ..., parse_dates: bool = ..., date_parser: Optional[Any] = ..., thousands: Optional[Any] = ..., comment: Optional[Any] = ..., skipfooter: int = ..., convert_float: bool = ..., mangle_dupe_cols: bool = ..., **kwds: Any) -> Any: ...
    @property
    def book(self) -> Any: ...
    @property
    def sheet_names(self) -> Any: ...
    def close(self) -> None: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None: ...
    def __del__(self) -> None: ...



from pandas.core.construction import create_series_with_explicit_dtype as create_series_with_explicit_dtype

from pandas.errors import AbstractMethodError as AbstractMethodError, EmptyDataError as EmptyDataError
from pandas.io.common import is_url as is_url, urlopen as urlopen, validate_header_arg as validate_header_arg
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.io.parsers import TextParser as TextParser
from typing import Any, Optional

class _HtmlFrameParser:
    io: Any = ...
    match: Any = ...
    attrs: Any = ...
    encoding: Any = ...
    displayed_only: Any = ...
    def __init__(self, io: Any, match: Any, attrs: Any, encoding: Any, displayed_only: Any) -> None: ...
    def parse_tables(self) -> Any: ...

class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class _LxmlFrameParser(_HtmlFrameParser):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

def read_html(io: Any, match: str = ..., flavor: Optional[Any] = ..., header: Optional[Any] = ..., index_col: Optional[Any] = ..., skiprows: Optional[Any] = ..., attrs: Optional[Any] = ..., parse_dates: bool = ..., thousands: str = ..., encoding: Optional[Any] = ..., decimal: str = ..., converters: Optional[Any] = ..., na_values: Optional[Any] = ..., keep_default_na: bool = ..., displayed_only: bool = ...) -> Any: ...



from pandas.core.dtypes.cast import maybe_downcast_to_dtype as maybe_downcast_to_dtype
from pandas.core.dtypes.common import ensure_object as ensure_object, is_datetime_or_timedelta_dtype as is_datetime_or_timedelta_dtype, is_decimal as is_decimal, is_number as is_number, is_numeric_dtype as is_numeric_dtype, is_scalar as is_scalar
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from typing import Any, Optional

def to_numeric(arg: Any, errors: str = ..., downcast: Optional[Any] = ...) -> Any: ...



from pandas.core.internals.blocks import Block as Block, BoolBlock as BoolBlock, CategoricalBlock as CategoricalBlock, ComplexBlock as ComplexBlock, DatetimeBlock as DatetimeBlock, DatetimeTZBlock as DatetimeTZBlock, ExtensionBlock as ExtensionBlock, FloatBlock as FloatBlock, IntBlock as IntBlock, ObjectBlock as ObjectBlock, TimeDeltaBlock as TimeDeltaBlock, _block_shape as _block_shape, _safe_reshape as _safe_reshape, make_block as make_block
from pandas.core.internals.managers import BlockManager as BlockManager, SingleBlockManager as SingleBlockManager, _transform_index as _transform_index, concatenate_block_managers as concatenate_block_managers, create_block_manager_from_arrays as create_block_manager_from_arrays, create_block_manager_from_blocks as create_block_manager_from_blocks



from pandas.core.indexes.api import CategoricalIndex as CategoricalIndex, DatetimeIndex as DatetimeIndex, Float64Index as Float64Index, Index as Index, Int64Index as Int64Index, IntervalIndex as IntervalIndex, InvalidIndexError as InvalidIndexError, MultiIndex as MultiIndex,  NumericIndex as NumericIndex, PeriodIndex as PeriodIndex, RangeIndex as RangeIndex, TimedeltaIndex as TimedeltaIndex, UInt64Index as UInt64Index, ensure_index as ensure_index, ensure_index_from_sequences as ensure_index_from_sequences, get_objs_combined_axis as get_objs_combined_axis



from pandas import Categorical as Categorical, DataFrame as DataFrame, Index as Index, MultiIndex as MultiIndex, Series as Series
from pandas._typing import FrameOrSeries as FrameOrSeries
from pandas.core import groupby as groupby
from pandas.core.dtypes.common import ensure_float64 as ensure_float64, ensure_int64 as ensure_int64, ensure_object as ensure_object, is_array_like as is_array_like, is_bool as is_bool, is_bool_dtype as is_bool_dtype, is_categorical_dtype as is_categorical_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_dtype_equal as is_dtype_equal, is_extension_array_dtype as is_extension_array_dtype, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_list_like as is_list_like, is_number as is_number, is_numeric_dtype as is_numeric_dtype, is_object_dtype as is_object_dtype, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna, na_value_for_dtype as na_value_for_dtype
from pandas.core.internals import concatenate_block_managers as concatenate_block_managers
from pandas.core.sorting import is_int64_overflow_possible as is_int64_overflow_possible
from pandas.errors import MergeError as MergeError
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Union

def merge(left: Any, right: Any, how: str=..., on: Any = ..., left_on: Any = ..., right_on: Any = ..., left_index: bool=..., right_index: bool=..., sort: bool=..., suffixes: Any = ..., copy: bool=..., indicator: bool=..., validate: Any = ...) -> DataFrame: ...
def merge_ordered(left: Any, right: Any, on: Any = ..., left_on: Any = ..., right_on: Any = ..., left_by: Any = ..., right_by: Any = ..., fill_method: Any = ..., suffixes: Any = ..., how: str=...) -> DataFrame: ...
def merge_asof(left: Any, right: Any, on: Any = ..., left_on: Any = ..., right_on: Any = ..., left_index: bool=..., right_index: bool=..., by: Any = ..., left_by: Any = ..., right_by: Any = ..., suffixes: Any = ..., tolerance: Any = ..., allow_exact_matches: bool=..., direction: str=...) -> DataFrame: ...

class _MergeOperation:
    left: Any = ...
    right: Any = ...
    how: Any = ...
    axis: Any = ...
    on: Any = ...
    left_on: Any = ...
    right_on: Any = ...
    copy: Any = ...
    suffixes: Any = ...
    sort: Any = ...
    left_index: Any = ...
    right_index: Any = ...
    indicator: Any = ...
    indicator_name: Any
    def __init__(self, left: Union[Series, DataFrame], right: Union[Series, DataFrame], how: str=..., on: Any = ..., left_on: Any = ..., right_on: Any = ..., axis: Any = ..., left_index: bool=..., right_index: bool=..., sort: bool=..., suffixes: Any = ..., copy: bool=..., indicator: bool=..., validate: Any = ...) -> None: ...
    def get_result(self) -> Any: ...

class _OrderedMerge(_MergeOperation):
    fill_method: Any = ...
    def __init__(self, left: Any, right: Any, on: Any = ..., left_on: Any = ..., right_on: Any = ..., left_index: bool=..., right_index: bool=..., axis: Any = ..., suffixes: Any = ..., copy: bool=..., fill_method: Any = ..., how: str=...) -> None: ...
    def get_result(self) -> Any: ...

class _AsOfMerge(_OrderedMerge):
    by: Any = ...
    left_by: Any = ...
    right_by: Any = ...
    tolerance: Any = ...
    allow_exact_matches: Any = ...
    direction: Any = ...
    def __init__(self, left: Any, right: Any, on: Any = ..., left_on: Any = ..., right_on: Any = ..., left_index: bool=..., right_index: bool=..., by: Any = ..., left_by: Any = ..., right_by: Any = ..., axis: Any = ..., suffixes: Any = ..., copy: bool=..., fill_method: Any = ..., how: str=..., tolerance: Any = ..., allow_exact_matches: bool=..., direction: str=...) -> None: ...



from pandas._config import get_option as get_option
from pandas.core.base import PandasObject as PandasObject

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Optional

def hist_series(self: Any, by: Optional[Any] = ..., ax: Optional[Any] = ..., grid: bool = ..., xlabelsize: Optional[Any] = ..., xrot: Optional[Any] = ..., ylabelsize: Optional[Any] = ..., yrot: Optional[Any] = ..., figsize: Optional[Any] = ..., bins: int = ..., backend: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def hist_frame(data: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., grid: bool = ..., xlabelsize: Optional[Any] = ..., xrot: Optional[Any] = ..., ylabelsize: Optional[Any] = ..., yrot: Optional[Any] = ..., ax: Optional[Any] = ..., sharex: bool = ..., sharey: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., bins: int = ..., backend: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def boxplot(data: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., ax: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., return_type: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def boxplot_frame(self: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., ax: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., return_type: Optional[Any] = ..., backend: Optional[Any] = ..., **kwargs: Any) -> Any: ...
def boxplot_frame_groupby(grouped: Any, subplots: bool = ..., column: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., ax: Optional[Any] = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., sharex: bool = ..., sharey: bool = ..., backend: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class PlotAccessor(PandasObject):
    def __init__(self, data: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def line(self, x: Optional[Any] = ..., y: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def bar(self, x: Optional[Any] = ..., y: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def barh(self, x: Optional[Any] = ..., y: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def box(self, by: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def hist(self, by: Optional[Any] = ..., bins: int = ..., **kwargs: Any) -> Any: ...
    def kde(self, bw_method: Optional[Any] = ..., ind: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    density: Any = ...
    def area(self, x: Optional[Any] = ..., y: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def pie(self, **kwargs: Any) -> Any: ...
    def scatter(self, x: Any, y: Any, s: Optional[Any] = ..., c: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def hexbin(self, x: Any, y: Any, C: Optional[Any] = ..., reduce_C_function: Optional[Any] = ..., gridsize: Optional[Any] = ..., **kwargs: Any) -> Any: ...



from typing import Any

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries, ABCSparseArray as ABCSparseArray
from pandas.core.ops.roperator import radd as radd, rand_ as rand_, rdivmod as rdivmod, rfloordiv as rfloordiv, rmod as rmod, rmul as rmul, ror_ as ror_, rpow as rpow, rsub as rsub, rtruediv as rtruediv, rxor as rxor

def add_special_arithmetic_methods(cls: Any) -> Any: ...
def add_flex_arithmetic_methods(cls: Any) -> None: ...



from collections import abc
from typing import Any, Optional

class XportReader(abc.Iterator[Any]):
    __doc__: Any = ...
    filepath_or_buffer: Any = ...
    def __init__(self, filepath_or_buffer: Any, index: Optional[Any] = ..., encoding: str = ..., chunksize: Optional[Any] = ...) -> None: ...
    def close(self) -> None: ...
    def __next__(self) -> Any: ...
    def get_chunk(self, size: Optional[Any] = ...) -> Any: ...
    def read(self, nrows: Optional[Any] = ...) -> Any: ...



import numpy as np
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.series import Series as Series
from typing import Any, Optional

class _Unstacker:
    values: Any = ...
    value_columns: Any = ...
    fill_value: Any = ...
    constructor: Any = ...
    index: Any = ...
    level: Any = ...
    lift: Any = ...
    new_index_levels: Any = ...
    new_index_names: Any = ...
    removed_name: Any = ...
    removed_level: Any = ...
    removed_level_full: Any = ...
    def __init__(self, values: np.ndarray, index: Any, level: Any = ..., value_columns: Any = ..., fill_value: Any = ..., constructor: Any = ...) -> None: ...
    def get_result(self) -> Any: ...
    def get_new_values(self) -> Any: ...
    def get_new_columns(self) -> Any: ...
    def get_new_index(self) -> Any: ...

def unstack(obj: Any, level: Any, fill_value: Optional[Any] = ...) -> Any: ...
def stack(frame: Any, level: int = ..., dropna: bool = ...) -> Any: ...
def stack_multiple(frame: Any, level: Any, dropna: bool = ...) -> Any: ...
def get_dummies(data: Any, prefix: Any = ..., prefix_sep: Any = ..., dummy_na: Any = ..., columns: Any = ..., sparse: Any = ..., drop_first: Any = ..., dtype: Any = ...) -> DataFrame: ...



from pandas.core.window.common import WindowGroupByMixin as WindowGroupByMixin
from pandas.core.window.rolling import _Rolling_and_Expanding
from typing import Any, Optional

class Expanding(_Rolling_and_Expanding):
    def __init__(self, obj: Any, min_periods: int = ..., center: bool = ..., axis: int = ..., **kwargs: Any) -> None: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def count(self, **kwargs: Any) -> Any: ...
    def apply(self, func: Any, raw: bool = ..., args: Any = ..., kwargs: Any = ...) -> Any: ... # type: ignore
    def sum(self, *args: Any, **kwargs: Any) -> Any: ...
    def max(self, *args: Any, **kwargs: Any) -> Any: ...
    def min(self, *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def median(self, **kwargs: Any) -> Any: ...
    def std(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def skew(self, **kwargs: Any) -> Any: ...
    def kurt(self, **kwargs: Any) -> Any: ...
    def quantile(self, quantile: Any, interpolation: str = ..., **kwargs: Any) -> Any: ...
    def cov(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., ddof: int = ..., **kwargs: Any) -> Any: ...
    def corr(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class ExpandingGroupby(WindowGroupByMixin, Expanding): ...
